from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.domain.balance.balance import Erc20Balance, NotEnoughNativeBalance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.web3_utils import get_amount_to_bridge_usdc

from web3_wizzard_lib.core.contract.stargate_router import StargateRouter


def bridge_usdc(native_balance, account, bridge_amount_interval, from_chain_config, to_chain_config, web3):
    if (from_chain_config['chain'] == 'FANTOM' and to_chain_config['chain'] == 'BASE') or (
            to_chain_config['chain'] == 'FANTOM' and from_chain_config['chain'] == 'BASE'):
        raise Exception('Stargate does not allow FANTOM<>BASE USDC transfer ')
    erc20_token = Erc20Token(from_chain_config['chain'], 'USDC', web3)

    token_balance = erc20_token.balance(account)

    logger.info(f"Token balance: {token_balance}")

    if token_balance.wei == 0:
        raise Exception(f"USDC balance is 0")

    amount_to_bridge = token_balance if bridge_amount_interval == 'all_balance' else Erc20Balance(
        get_amount_to_bridge_usdc(bridge_amount_interval),
        native_balance.chain,
        'USDC'
    )

    if amount_to_bridge.wei > token_balance.wei:
        raise NotEnoughNativeBalance(f"Account balance ({amount_to_bridge}) < {amount_to_bridge} amount to bridge.")

    logger.info(f"Bridging: {amount_to_bridge}")

    erc20_token = Erc20Token(from_chain_config['chain'], "USDC", web3)

    chain_contracts = get_contracts_for_chain(from_chain_config['chain'])

    stargate_router_address = chain_contracts["STARGATE_ROUTER"]

    if erc20_token.allowance(account, stargate_router_address) < amount_to_bridge.wei:
        erc20_token.allowance(account, stargate_router_address)

    stargate_router = StargateRouter(chain_contracts["STARGATE_ROUTER"], web3)
    native_fee_wei = stargate_router.count_native_fee_stargate(to_chain_config['stargate_chain_id'], account.address)

    args = [account, native_fee_wei, amount_to_bridge, to_chain_config]

    stargate_router.swap(*args)



import requests
from eth_account import Account
from eth_account.messages import encode_defunct
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.rubyscore_contract import RubyscoreContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class Rubyscore(SubModule):
    module_name = 'RUBYSCORE'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        message = " A signature is required for authorization on the platform and does not pose a threat to users!"

        message_encoded = encode_defunct(text=message)
        signed_message = Account.sign_message(message_encoded, private_key=account.key).signature.hex()

        login_request_url = f"https://rubyscore.io/api/auth/login?signature={signed_message}&message=+A+signature+is+required+for+authorization+on+the+platform+and+does+not+pose+a+threat+to+users!&wallet={account.address}"

        login_token = requests.post(login_request_url).json()['result']['token']

        request_signature_request = "https://rubyscore.io/api/attestation/claim?project=linea"
        request_signature_response = requests.post(
            request_signature_request,
            headers={
                'Authorization': f'Bearer {login_token}'
            }
        ).json()['result']

        schemaId = request_signature_response['attestationParams']['schemaId']
        expirationDate = int(request_signature_response['attestationParams']['expirationDate'])
        signature = request_signature_response['signature']

        contract_address = get_contracts_for_chain(chain)['RUBYSCORE']
        rubyscore = RubyscoreContract(contract_address, web3)
        rubyscore.attest_rubyscore(account, signature, schemaId, expirationDate)

    def log(self):
        return "RUBYSCORE"



import random

from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance_utils import verify_balance, amount_to_swap_from_interval
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.module.module import Module
from sybil_engine.utils.utils import SwapException
from sybil_engine.utils.validation_utils import validate_amount_interval, validate_token
from sybil_engine.utils.web3_utils import init_web3
from web3 import Web3

from web3_wizzard_lib.core.modules.swap.swap_list import swap_facade


class ConcreteSwap(Module):
    module_name = 'CONCRETE_SWAP'
    module_config = 'swap_config'
    allowed_chains = ['ZKSYNC', 'LINEA', 'BASE', 'SCROLL', 'ARBITRUM']
    sleep_after_var = True

    def execute(self, chain, amount_interval, from_token, to_token, swap_app, account):
        if isinstance(swap_app, list):
            swap_app = random.choice(swap_app)

        self.sleep_after_var = True
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        native_balance = verify_balance(self.min_native_balance, chain_instance, account, web3)

        from_token_contract = Erc20Token(chain, from_token, web3)
        to_token_contract = Erc20Token(chain, to_token, web3)

        if Web3.is_address(from_token):
            from_token_symbol = from_token_contract.symbol()
        else:
            from_token_symbol = from_token

        if Web3.is_address(to_token):
            to_token_symbol = to_token_contract.symbol()
        else:
            to_token_symbol = to_token

        pair_to_swap = self.create_pair_to_swap(
            from_token_symbol,
            swap_app,
            to_token_symbol,
        )

        amount_to_swap = amount_to_swap_from_interval(
            account,
            chain,
            self.min_native_balance,
            native_balance,
            amount_interval,
            from_token,
            web3
        )

        if amount_to_swap.wei < 100000:
            logger.info("Low balance, ignore")
            return

        try:
            swap_facade.swap(
                account,
                amount_to_swap,
                chain_instance,
                pair_to_swap,
                swap_app,
                from_token,
                to_token,
                web3
            )
        except SwapException as e:
            logger.info(e.message)
            self.sleep_after_var = False

    def create_pair_to_swap(self, from_token, swap_app, to_token):
        pair_to_swap = {
            'name': f'{from_token}>{to_token}',
            'tokens': [from_token, to_token],
            'slippage': 2,
            'app': swap_app
        }
        return pair_to_swap

    def log(self):
        return "SWAP"

    def sleep_after(self):
        return self.sleep_after_var

    def parse_params(self, module_params):
        self.validate_supported_chain(module_params['chain'])
        validate_amount_interval(module_params['amount_interval'])
        validate_token(module_params['from_token'])
        validate_token(module_params['to_token'])
        #validate_dex(module_params['app'])

        return (
            module_params['chain'],
            module_params['amount_interval'],
            module_params['from_token'],
            module_params['to_token'],
            module_params['app']
        )



import time

from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/omnisea.json")


class OmniSea(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def create(self, account, title, symbol):
        txn_params = self.build_generic_data(account.address, False)

        contract_txn = self.contract.functions.create([
            title,
            symbol,
            "",
            "",
            0,
            True,
            0,
            int(time.time()) + 1000000]
        ).build_transaction(txn_params)

        return contract_txn



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.data.networks import get_ids_chain
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/layerbank.json")


class LayerBankContract(Contract):
    def __init__(self, contract_address, web3):
        self.weth_token = get_tokens_for_chain(get_ids_chain()[web3.eth.chain_id])['WETH']
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def supply(self, account, token, amount):
        txn_params = self.build_generic_data(account.address, False)

        contract_txn = self.contract.functions.supply(token, amount).build_transaction(txn_params)

        return contract_txn

    @evm_transaction
    def enter_markets(self, account, token):
        txn_params = self.build_generic_data(account.address, False)

        return self.contract.functions.enterMarkets([token]).build_transaction(txn_params)

    @evm_transaction
    def borrow(self, account, token, amount):
        txn_params = self.build_generic_data(account.address, False)

        contract_txn = self.contract.functions.borrow(token, amount).build_transaction(txn_params)

        return contract_txn

    @evm_transaction
    def repay_borrow(self, account, token, amount):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = amount

        contract_txn = self.contract.functions.repayBorrow(token, amount).build_transaction(txn_params)

        return contract_txn

    @evm_transaction
    def redeem_token(self, account, token, amount):
        txn_params = self.build_generic_data(account.address, False)

        contract_txn = self.contract.functions.redeemUnderlying(token, amount).build_transaction(txn_params)

        return contract_txn

    def market_list_of(self, account):
        return self.contract.functions.marketListOf(account.address).call()

    def account_liquidity_of(self, account):
        return self.contract.functions.accountLiquidityOf(account.address).call()



import secrets

import base58
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.imagine_contract import ImagineContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class Imagine(SubModule):
    module_name = 'IMAGINE'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['IMAGINE']
        imagine_contract = ImagineContract(contract_address, web3)

        random_bytes = secrets.token_bytes(32)

        # Encode the byte string using base58
        base58_string = base58.b58encode(random_bytes).decode('utf-8')

        uri = f"https://ipfs.io/ipfs/{base58_string}/metadata.json"

        imagine_contract.mint(account, uri)

    def log(self):
        return "IMAGINE NFT"



from sybil_engine.domain.dex import Dex
from sybil_engine.domain.swap_facade import SwapFacade
from sybil_engine.utils.package_import_utils import get_all_subclasses, import_all_modules_in_directory

import web3_wizzard_lib.core.modules.swap

def get_swap_classes():
    import_all_modules_in_directory("web3_wizzard_lib.core.modules.swap")
    return get_all_subclasses(Dex)

swap_facade = SwapFacade(get_swap_classes())



from sybil_engine.domain.dex import Dex

from web3_wizzard_lib.core.contract.syncswap_classic_pool_factory import SyncSwapClassicPoolFactory
from web3_wizzard_lib.core.contract.syncswap_pool import SyncSwapPoolContract
from web3_wizzard_lib.core.contract.syncswap_router import SyncSwapRouter


class Syncswap(Dex):
    dex_name = 'syncswap'
    swap_contract = 'SYNCSWAP'
    supported_chains = ['ZKSYNC', 'LINEA', 'SCROLL']

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.weth_address = self.tokens['WETH']
        self.pool_classic_pool_factory_address = self.chain_contracts['SYNCSWAP_CLASSIC_POOL_FACTORY']
        self.syncswap_contract_address = self.chain_contracts[self.swap_contract]
        self.syncswap_router = SyncSwapRouter(self.syncswap_contract_address, self.web3)

    def swap_native_for_token(self, account, amount_to_swap, slippage, to_token_address):
        token_in_data_address = self.weth_address
        token_in_address = self.tokens['ZERO_ADDRESS']

        amount_out_min = int(
            self.get_amount_out_min(account, amount_to_swap, token_in_data_address, to_token_address) * slippage
        )

        args = [
            account,
            amount_to_swap,
            self.weth_address,
            token_in_address,
            self.get_pool_address(token_in_data_address, to_token_address),
            amount_out_min
        ]

        return args, self.syncswap_router.swap

    def swap_token_for_native(self, account, amount_to_swap, from_token_address, slippage):
        token_in_data_address = self.tokens[amount_to_swap.token]
        token_in_address = self.tokens[amount_to_swap.token]

        amount_out_min = int(
            self.get_amount_out_min(account, amount_to_swap, token_in_data_address, self.weth_address) * slippage)

        args = [account, amount_to_swap, token_in_address, token_in_data_address, self.get_pool_address(token_in_data_address, self.weth_address), amount_out_min]

        return args, self.syncswap_router.swap

    def get_amount_out_min(self, account, amount_to_swap, token_in_data_address, to_token_address):
        pool_address = self.get_pool_address(token_in_data_address, to_token_address)

        if isinstance(pool_address, str):
            syncswap_pool = SyncSwapPoolContract(pool_address, self.web3)
        else:
            syncswap_pool = SyncSwapPoolContract(list(pool_address.items())[0][1], self.web3)

        return syncswap_pool.get_amount_out(account, amount_to_swap, token_in_data_address)

    def get_pool_address(self, token_in_data_address, to_token_address):
        pool_factory = SyncSwapClassicPoolFactory(self.pool_classic_pool_factory_address, self.web3)
        return pool_factory.get_pool(token_in_data_address, to_token_address)



from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NotEnoughNativeBalance
from sybil_engine.domain.balance.balance_utils import interval_to_native_balance, verify_balance
from sybil_engine.utils.utils import ConfigurationException
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.stargate_v2_contract import StargateV2Contract
from web3_wizzard_lib.core.modules.bridge.bridge import Bridge

chain_ids = {
    'LINEA': 30183,
    'SCROLL': 30214,
    'OPTIMISM': 30111,
    'ARBITRUM': 30110,
    'BASE': 30184
}


class StargateV2SubModule(Bridge):
    supported_tokens = ['NATIVE']

    def execute(self, bridge_amount_interval, from_chain, to_chain, token, min_native_balance, account):
        if token != 'NATIVE':
            raise ConfigurationException('Currently only native token bridge is supported by Stargate V2')

        from_chain_instance = get_chain_instance(from_chain)
        web3 = init_web3(from_chain_instance, account.proxy)
        stargate_v2_contract = get_contracts_for_chain(from_chain)['STARGATE_V2']

        amount = self.__calculate_bridge_amount(
            account,
            bridge_amount_interval,
            from_chain_instance,
            min_native_balance,
            web3
        )

        logger.info(f"Bridge {from_chain} –> {to_chain} | {amount}")

        send_params = (
            chain_ids[to_chain],
            f'0x000000000000000000000000{account.address[2:]}',
            amount.wei,  # amountLD,
            int(amount.wei * 0.995),  # minAmountLd,
            b'',
            b'',
            b'\x01',
        )

        stargate_v2 = StargateV2Contract(stargate_v2_contract, web3)

        stargate_fee = stargate_v2.quote_send(send_params)

        stargate_v2.send(
            account,
            amount.wei + stargate_fee[0],
            send_params,
            stargate_fee,
            account.address
        )

    def __calculate_bridge_amount(self, account, bridge_amount_interval, from_chain_instance, min_native_balance, web3):
        native_without_min = verify_balance(min_native_balance, from_chain_instance, account, web3)

        amount = interval_to_native_balance(bridge_amount_interval, account, from_chain_instance['chain'], web3)
        if amount.wei > native_without_min.wei:
            if bridge_amount_interval == 'all_balance':
                amount = native_without_min
            else:
                raise NotEnoughNativeBalance(
                    f"The account balance ({native_without_min}) < bridging amount ({amount}).")
        return amount



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/woofi.json")


class WoofiSwap(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def swap(self, account, amount_to_swap, token_in_address, token_out_address, amount_out_min):
        sender = account.address
        txn_params = self.build_generic_data(sender, False)

        txn_params['value'] = amount_to_swap.wei if amount_to_swap.token == 'ETH' else 0

        return self.contract.functions.swap(
            token_in_address,
            token_out_address,
            amount_to_swap.wei,
            amount_out_min,
            sender,
            sender
        ).build_transaction(txn_params)

    def query_swap(self, from_token, to_token, from_amount):
        return self.contract.functions.querySwap(from_token, to_token, from_amount.wei).call()



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction, l0_evm_transaction

from web3_wizzard_lib.core.modules.stargate.stargate_gas_checker import stargate_check_gas

with open("resources/abi/stargate_router.json") as f:
    abi = f.read()


class StargateRouter(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def add_liquidity(self, account, pool_amount):
        from_address = account.address

        txn_params = self.build_generic_data(from_address, False)

        contract_txn = self.contract.functions.addLiquidity(
            self.chain_instance['stargate_usdc_pool'],
            pool_amount.wei,
            from_address
        ).build_transaction(txn_params)

        contract_txn['gas'] = (self.web3.eth.estimate_gas(contract_txn) * 1.1)

        return contract_txn

    @evm_transaction
    def instant_redeem(self, account, amount_to_bridge, stargate_pool_id):
        from_address = account.address

        txn_params = self.build_generic_data(from_address, False)

        contract_txn = self.contract.functions.instantRedeemLocal(
            stargate_pool_id,
            amount_to_bridge,
            from_address
        ).build_transaction(txn_params)
        contract_txn['gas'] = int(self.web3.eth.estimate_gas(contract_txn) * 1.1)

        return contract_txn

    @stargate_check_gas("USDC")
    @l0_evm_transaction
    def swap(self, account, value_fee, amount_to_bridge, to_chain_instance):
        from_address = account.address

        txn_params = self.build_generic_data(from_address, False)
        txn_params['value'] = value_fee

        lz_tx_obj = [0, 0, '0x0000000000000000000000000000000000000001']

        if amount_to_bridge.token == 'USDC':
            stargate_pool = to_chain_instance['stargate_usdc_pool']
        else:
            stargate_pool = to_chain_instance['stargate_eth_pool']

        contract_txn = self.contract.functions.swap(
            to_chain_instance['stargate_chain_id'],
            stargate_pool,
            stargate_pool,
            from_address,
            amount_to_bridge.wei,
            int(amount_to_bridge.wei * 0.985),
            lz_tx_obj,
            from_address,
            '0x'
        ).build_transaction(txn_params)
        contract_txn['gas'] = int(self.web3.eth.estimate_gas(contract_txn) * 1.1)

        return contract_txn

    def count_native_fee_stargate(self, chain_id, address):
        return int(
            self.contract.functions.quoteLayerZeroFee(
                chain_id,
                1,
                address,
                '0x',
                (0, 0, address)
            ).call()[0] * 1.1
        )



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/zks.json")


class ZKS(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def register(self, account, domain_name):
        sender = account.address

        txn_params = self.build_generic_data(sender, False)

        contract_txn = self.contract.functions.register(domain_name, sender, 1).build_transaction(txn_params)
        contract_txn['gas'] = self.web3.eth.estimate_gas(contract_txn)

        return contract_txn

    def available(self, domain_name):
        return self.contract.functions.available(domain_name).call()



from sybil_engine.domain.dex import Dex

from web3_wizzard_lib.core.contract.pancake_quoter import PancakeQuoter
from web3_wizzard_lib.core.contract.pancake_router import PancakeRouter


class Pancake(Dex):
    dex_name = 'pancake'
    swap_contract = "PANCAKE_ROUTER"
    supported_chains = ['ZKSYNC', 'BASE', 'LINEA', 'ARBITRUM']

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.weth_address = self.tokens['WETH']
        self.pancake_router_contract = self.chain_contracts[self.swap_contract]
        self.pancake_router = PancakeRouter(self.pancake_router_contract, self.web3)

    def swap_native_for_token(self, account, amount_to_swap, slippage, to_token_address):

        amount_out_min = int(self.get_amount_out_min(amount_to_swap, self.weth_address, to_token_address) * slippage)
        args = [account, amount_to_swap, self.weth_address, to_token_address, amount_out_min]
        func = self.pancake_router.swap_to_token

        return args, func

    def swap_token_for_native(self, account, amount_to_swap, from_token_address, slippage):
        amount_out_min = int(
            self.get_amount_out_min(amount_to_swap, from_token_address, self.weth_address) * slippage)

        args = [account, amount_to_swap, from_token_address, self.weth_address, amount_out_min]
        func = self.pancake_router.swap_to_eth
        return args, func

    def swap_token_for_token(self, account, amount_to_swap, slippage, from_token_address, to_token_address):
        raise Exception('Not supported')

    def get_amount_out_min(self, amount_to_swap, from_token_address, to_token_address):
        pancake_quoter_contract = self.chain_contracts["PANCAKE_QUOTER"]
        pancake_quoter = PancakeQuoter(pancake_quoter_contract, self.web3)

        return pancake_quoter.quote_exact_input_single(from_token_address, to_token_address, amount_to_swap.wei)



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/layerbank_token.json")

MAX_ALLOWANCE = 115792089237316195423570985008687907853269984665640564039457584007913129639935


class LayerBankToken(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def balance_of(self, account):
        return self.contract.functions.balanceOf(account.address).call()

    def underlying_balance_of(self, account):
        return self.contract.functions.balanceOf(account.address).call()

    def exchange_rate(self):
        return self.contract.functions.exchangeRate().call()

    def decimals(self):
        return self.contract.functions.decimals().call()

    def one_decimal(self):
        return 10 ** self.contract.functions.decimals().call()

    def symbol(self):
        return self.contract.functions.symbol().call()



from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.module.module import Module
from sybil_engine.utils.utils import ConfigurationException
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.new_rage_contract import NewRageContract


class NewRageModule(Module):
    module_name = 'NEW_RAGE_WITHDRAW'
    module_config = 'new_rage_config'

    def execute(self, token, account, chain='ARBITRUM'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, None)

        if token == 'USDC':
            contract_address = "0xf9305009FbA7E381b3337b5fA157936d73c2CF36"
        elif token == 'GLP':
            contract_address = "0x8478AB5064EbAC770DdCE77E7D31D969205F041E"
        else:
            raise ConfigurationException("No such token supported by Rage withdraw module")
        rage = NewRageContract(contract_address, web3)

        #balance = rage.balance_of(account)
        shares = rage.max_redeem(account)

        if shares > 0:
            logger.info(f"Withdraw  {shares} shares {token}")
            rage.redeem(account, shares)
        else:
            logger.info(f"Account {account.address} has 0 balance in Rage")

        #add_accumulator("Total rage withdraw", balance)

    def log(self):
        return "RAGE WITHDRAW"

    def parse_params(self, module_params):
        return module_params['token'],



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/satoshi_universe.json")


class SatoshiUniverseContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def mint(self, account, amount):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = 150000000000000

        contract_txn = self.contract.functions.mint(
            (
                account.address,
                '0xFf9B136F19D1Ad1f1716e2Ebe98b4A773f263275',
                amount,
                [],
                1,
                '0x'
            )
        ).build_transaction(txn_params)

        return contract_txn



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/townstory.json")


class TownstoryContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def create_account_sign(self, account, signature, deadline):
        txn_params = self.build_generic_data(account.address, False)

        contract_txn = self.contract.functions.createAccountSign(
            signature, 0, deadline
        ).build_transaction(txn_params)

        return contract_txn



from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance_utils import get_native_balance, interval_to_eth_balance, \
    interval_to_erc20_balance
from sybil_engine.module.module import Order, Module
from sybil_engine.utils.app_account_utils import AppAccount
from sybil_engine.utils.retry import retry
from sybil_engine.utils.utils import ConfigurationException
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules.bank.basilisk import Basilisk
from web3_wizzard_lib.core.modules.bank.eralend import Eralend
from web3_wizzard_lib.core.modules.bank.layerbank import LayerBank
from web3_wizzard_lib.core.modules.bank.mendi_finance import MendiFinance
from web3_wizzard_lib.core.modules.bank.aave import Aave
from web3_wizzard_lib.core.modules.bank.reactorfusion import ReactorFusion
from web3_wizzard_lib.core.modules.bank.zerolend import ZeroLend


class Banking(Module):
    module_name = 'BANKING'
    random_order = Order.RANDOM
    module_config = 'banking_config'

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def execute(self, bank_app_name, action, token, amount_interval, chain, account: AppAccount):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)
        balance = get_native_balance(account, web3, chain_instance)

        logger.info(f"Balance {balance}")

        if token == 'ETH':
            amount = interval_to_eth_balance(amount_interval, account, chain, web3)
        else:
            amount = interval_to_erc20_balance(amount_interval, account, token, chain, web3)

        bank_app = self.get_bank_app(bank_app_name, chain, web3)

        logger.info(f"{bank_app.app_name}")

        self.perform_action(bank_app, action, amount, token, account)

    def perform_action(self, bank_app, action, amount, token, account):
        logger.info(action)
        if action == 'SUPPLY':
            bank_app.supply(account, amount)
        elif action == 'BORROW':
            bank_app.borrow(account, amount)
        elif action == 'REPAY':
            bank_app.repay_borrow(account, amount)
        elif action == 'REDEEM':
            amount = bank_app.get_deposit_amount(account, token)
            logger.info(f"Redeem {amount} of {token} from {bank_app.app_name}")

            bank_app.redeem(account, amount, token)
        else:
            raise ConfigurationException("Unsupported action")

    def get_bank_app(self, bank_app, chain, web3):
        bank_app = self.get_bank_app_by_name(bank_app)

        if chain not in bank_app.supported_chains:
            raise ConfigurationException(
                f"{bank_app} not supported in {chain}. Supported chains: {bank_app.supported_chains}")

        return bank_app(chain, web3)

    def get_bank_app_by_name(self, bank_app_name):
        for bank_app in self.get_bank_apps():
            if bank_app.app_name == bank_app_name:
                return bank_app
        raise ConfigurationException(f"No Bank App with name {bank_app_name} found")

    def get_bank_apps(self):
        return {
            Basilisk,
            ReactorFusion,
            Eralend,
            MendiFinance,
            LayerBank,
            Aave,
            ZeroLend
        }

    def log(self):
        return "BANKING"

    def parse_params(self, module_params):
        if 'chain' not in module_params:
            module_params['chain'] = 'LINEA'

        return [
            module_params['bank'],
            module_params['action'],
            module_params['token'],
            module_params['amount_interval'],
            module_params['chain']
        ]



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/pancake/quoter.json")


class PancakeQuoter(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def quote_exact_input_single(self, from_token: str, to_token: str, amount: int):
        return self.contract.functions.quoteExactInputSingle(
            (
                from_token,
                to_token,
                amount,
                500,
                0
            )
        ).call()[0]



from loguru import logger
from sybil_engine.domain.dex import Dex
from sybil_engine.utils.retry import retry
from sybil_engine.utils.utils import AccountException

from web3_wizzard_lib.core.contract.odos import Odos


class OdosSwap(Dex):
    dex_name = 'odos'
    swap_contract = 'ODOS'
    supported_chains = ['ZKSYNC', 'BASE', 'LINEA', 'ARBITRUM']

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.odos_router_contract = self.chain_contracts[self.swap_contract]
        self.odos_router = Odos(self.odos_router_contract, self.web3)

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def swap(self, amount_to_swap, from_token, to_token, slippage, account):
        logger.info(
            f"Swap {amount_to_swap.log_line()}->{to_token.symbol()} in {self.dex_name} ({self.chain_instance['chain']})")

        if amount_to_swap.token == 'ETH':
            from_token_address = '0x0000000000000000000000000000000000000000'
        else:
            from_token_address = from_token.erc20_contract.contract_address

        if to_token.token == 'ETH':
            to_token_address = '0x0000000000000000000000000000000000000000'
        else:
            to_token_address = to_token.erc20_contract.contract_address

        quote_data = self.odos_router.quote(
            account, from_token_address, to_token_address, amount_to_swap.wei, slippage, self.chain_instance['chain']
        )

        if amount_to_swap.token == 'ETH':
            transaction_data = self.odos_router.assemble(account, quote_data["pathId"])

            func = self.odos_router.swap
        else:
            balance = from_token.balance(account)

            if balance.wei < amount_to_swap.wei:
                raise AccountException(f"Balance {balance} < {amount_to_swap}")

            if from_token.allowance(account, self.odos_router_contract) < amount_to_swap.wei:
                from_token.approve(account, self.odos_router_contract)

            transaction_data = self.odos_router.assemble(account, quote_data["pathId"])

            if to_token.token == 'ETH':
                func = self.odos_router.swap
            else:
                raise Exception('Not supported')

        func(account, transaction_data)



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction

abi = [{'inputs': [{'internalType': 'address', 'name': '_zroToken', 'type': 'address'},
                   {'internalType': 'address', 'name': '_endpoint', 'type': 'address'},
                   {'internalType': 'bytes32', 'name': '_merkleRoot', 'type': 'bytes32'},
                   {'internalType': 'address', 'name': '_donateContract', 'type': 'address'},
                   {'internalType': 'address', 'name': '_stargateUsdc', 'type': 'address'},
                   {'internalType': 'address', 'name': '_stargateUsdt', 'type': 'address'},
                   {'internalType': 'address', 'name': '_stargateNative', 'type': 'address'},
                   {'internalType': 'uint256', 'name': '_nativePrice', 'type': 'uint256'},
                   {'internalType': 'address', 'name': '_owner', 'type': 'address'}], 'stateMutability': 'nonpayable',
        'type': 'constructor'},
       {'inputs': [{'internalType': 'address', 'name': 'target', 'type': 'address'}], 'name': 'AddressEmptyCode',
        'type': 'error'}, {'inputs': [{'internalType': 'address', 'name': 'account', 'type': 'address'}],
                           'name': 'AddressInsufficientBalance', 'type': 'error'},
       {'inputs': [{'internalType': 'address', 'name': 'user', 'type': 'address'}], 'name': 'AlreadyClaimed',
        'type': 'error'}, {'inputs': [], 'name': 'DonateAndClaimAlreadySet', 'type': 'error'},
       {'inputs': [], 'name': 'FailedInnerCall', 'type': 'error'}, {
           'inputs': [{'internalType': 'enum Currency', 'name': 'currency', 'type': 'uint8'},
                      {'internalType': 'uint256', 'name': 'expectedAmount', 'type': 'uint256'},
                      {'internalType': 'uint256', 'name': 'actualAmount', 'type': 'uint256'}],
           'name': 'InsufficientDonation', 'type': 'error'}, {'inputs': [], 'name': 'InvalidDelegate', 'type': 'error'},
       {'inputs': [], 'name': 'InvalidEndpointCall', 'type': 'error'},
       {'inputs': [], 'name': 'InvalidNativeStargate', 'type': 'error'},
       {'inputs': [], 'name': 'InvalidProof', 'type': 'error'},
       {'inputs': [{'internalType': 'uint32', 'name': 'eid', 'type': 'uint32'}], 'name': 'NoPeer', 'type': 'error'},
       {'inputs': [], 'name': 'OnlyDonateAndClaim', 'type': 'error'},
       {'inputs': [{'internalType': 'address', 'name': 'addr', 'type': 'address'}], 'name': 'OnlyEndpoint',
        'type': 'error'}, {'inputs': [{'internalType': 'uint32', 'name': 'eid', 'type': 'uint32'},
                                      {'internalType': 'bytes32', 'name': 'sender', 'type': 'bytes32'}],
                           'name': 'OnlyPeer', 'type': 'error'},
       {'inputs': [{'internalType': 'address', 'name': 'owner', 'type': 'address'}], 'name': 'OwnableInvalidOwner',
        'type': 'error'}, {'inputs': [{'internalType': 'address', 'name': 'account', 'type': 'address'}],
                           'name': 'OwnableUnauthorizedAccount', 'type': 'error'},
       {'inputs': [{'internalType': 'address', 'name': 'token', 'type': 'address'}], 'name': 'SafeERC20FailedOperation',
        'type': 'error'}, {'inputs': [{'internalType': 'enum Currency', 'name': 'currency', 'type': 'uint8'}],
                           'name': 'UnsupportedCurrency', 'type': 'error'},
       {'inputs': [], 'name': 'DENOMINATOR', 'outputs': [{'internalType': 'uint256', 'name': '', 'type': 'uint256'}],
        'stateMutability': 'view', 'type': 'function'}, {'inputs': [{'components': [
        {'internalType': 'uint32', 'name': 'srcEid', 'type': 'uint32'},
        {'internalType': 'bytes32', 'name': 'sender', 'type': 'bytes32'},
        {'internalType': 'uint64', 'name': 'nonce', 'type': 'uint64'}], 'internalType': 'struct Origin',
        'name': 'origin', 'type': 'tuple'}],
           'name': 'allowInitializePath', 'outputs': [
            {'internalType': 'bool', 'name': '', 'type': 'bool'}], 'stateMutability': 'view', 'type': 'function'}, {
           'inputs': [{'internalType': 'enum Currency', 'name': '_currency', 'type': 'uint8'},
                      {'internalType': 'address', 'name': '_user', 'type': 'address'},
                      {'internalType': 'uint256', 'name': '_zroAmount', 'type': 'uint256'}], 'name': 'assertDonation',
           'outputs': [], 'stateMutability': 'view', 'type': 'function'}, {
           'inputs': [{'internalType': 'address', 'name': '_user', 'type': 'address'},
                      {'internalType': 'enum Currency', 'name': '_currency', 'type': 'uint8'},
                      {'internalType': 'uint256', 'name': '_zroAmount', 'type': 'uint256'},
                      {'internalType': 'bytes32[]', 'name': '_proof', 'type': 'bytes32[]'},
                      {'internalType': 'address', 'name': '_to', 'type': 'address'},
                      {'internalType': 'bytes', 'name': '', 'type': 'bytes'}], 'name': 'claim', 'outputs': [{
            'components': [
                {
                    'internalType': 'bytes32',
                    'name': 'guid',
                    'type': 'bytes32'},
                {
                    'internalType': 'uint64',
                    'name': 'nonce',
                    'type': 'uint64'},
                {
                    'components': [
                        {
                            'internalType': 'uint256',
                            'name': 'nativeFee',
                            'type': 'uint256'},
                        {
                            'internalType': 'uint256',
                            'name': 'lzTokenFee',
                            'type': 'uint256'}],
                    'internalType': 'struct MessagingFee',
                    'name': 'fee',
                    'type': 'tuple'}],
            'internalType': 'struct MessagingReceipt',
            'name': 'receipt',
            'type': 'tuple'}],
           'stateMutability': 'payable', 'type': 'function'}, {
           'inputs': [{'internalType': 'enum Currency', 'name': '_currency', 'type': 'uint8'},
                      {'internalType': 'uint256', 'name': '_zroAmount', 'type': 'uint256'},
                      {'internalType': 'bytes32[]', 'name': '_proof', 'type': 'bytes32[]'},
                      {'internalType': 'address', 'name': '_to', 'type': 'address'},
                      {'internalType': 'bytes', 'name': '', 'type': 'bytes'}], 'name': 'claim', 'outputs': [{
            'components': [
                {
                    'internalType': 'bytes32',
                    'name': 'guid',
                    'type': 'bytes32'},
                {
                    'internalType': 'uint64',
                    'name': 'nonce',
                    'type': 'uint64'},
                {
                    'components': [
                        {
                            'internalType': 'uint256',
                            'name': 'nativeFee',
                            'type': 'uint256'},
                        {
                            'internalType': 'uint256',
                            'name': 'lzTokenFee',
                            'type': 'uint256'}],
                    'internalType': 'struct MessagingFee',
                    'name': 'fee',
                    'type': 'tuple'}],
            'internalType': 'struct MessagingReceipt',
            'name': 'receipt',
            'type': 'tuple'}],
           'stateMutability': 'payable', 'type': 'function'}, {
           'inputs': [{'internalType': 'enum Currency', 'name': 'currency', 'type': 'uint8'},
                      {'internalType': 'uint256', 'name': 'amountToDonate', 'type': 'uint256'},
                      {'internalType': 'uint256', 'name': '_zroAmount', 'type': 'uint256'},
                      {'internalType': 'bytes32[]', 'name': '_proof', 'type': 'bytes32[]'},
                      {'internalType': 'address', 'name': '_to', 'type': 'address'},
                      {'internalType': 'bytes', 'name': '_extraBytes', 'type': 'bytes'}], 'name': 'donateAndClaim',
           'outputs': [{'components': [{'internalType': 'bytes32', 'name': 'guid', 'type': 'bytes32'},
                                       {'internalType': 'uint64', 'name': 'nonce', 'type': 'uint64'}, {'components': [
                   {'internalType': 'uint256', 'name': 'nativeFee', 'type': 'uint256'},
                   {'internalType': 'uint256', 'name': 'lzTokenFee', 'type': 'uint256'}],
                                           'internalType': 'struct MessagingFee',
                                           'name': 'fee',
                                           'type': 'tuple'}],
                        'internalType': 'struct MessagingReceipt', 'name': 'receipt', 'type': 'tuple'}],
           'stateMutability': 'payable', 'type': 'function'}, {'inputs': [], 'name': 'donateContract', 'outputs': [
        {'internalType': 'contract IDonate', 'name': '', 'type': 'address'}], 'stateMutability': 'view',
                                                               'type': 'function'}, {'inputs': [], 'name': 'endpoint',
                                                                                     'outputs': [{
                                                                                         'internalType': 'contract ILayerZeroEndpointV2',
                                                                                         'name': '',
                                                                                         'type': 'address'}],
                                                                                     'stateMutability': 'view',
                                                                                     'type': 'function'}, {'inputs': [{
        'components': [
            {
                'internalType': 'uint32',
                'name': 'srcEid',
                'type': 'uint32'},
            {
                'internalType': 'bytes32',
                'name': 'sender',
                'type': 'bytes32'},
            {
                'internalType': 'uint64',
                'name': 'nonce',
                'type': 'uint64'}],
        'internalType': 'struct Origin',
        'name': '',
        'type': 'tuple'},
        {
            'internalType': 'bytes',
            'name': '',
            'type': 'bytes'},
        {
            'internalType': 'address',
            'name': '_sender',
            'type': 'address'}],
           'name': 'isComposeMsgSender',
           'outputs': [{
               'internalType': 'bool',
               'name': '',
               'type': 'bool'}],
           'stateMutability': 'view',
           'type': 'function'},
       {'inputs': [{'components': [{'internalType': 'uint32', 'name': 'srcEid', 'type': 'uint32'},
                                   {'internalType': 'bytes32', 'name': 'sender', 'type': 'bytes32'},
                                   {'internalType': 'uint64', 'name': 'nonce', 'type': 'uint64'}],
                    'internalType': 'struct Origin', 'name': '_origin', 'type': 'tuple'},
                   {'internalType': 'bytes32', 'name': '_guid', 'type': 'bytes32'},
                   {'internalType': 'bytes', 'name': '_message', 'type': 'bytes'},
                   {'internalType': 'address', 'name': '_executor', 'type': 'address'},
                   {'internalType': 'bytes', 'name': '_extraData', 'type': 'bytes'}], 'name': 'lzReceive',
        'outputs': [], 'stateMutability': 'payable', 'type': 'function'},
       {'inputs': [], 'name': 'merkleRoot', 'outputs': [{'internalType': 'bytes32', 'name': '', 'type': 'bytes32'}],
        'stateMutability': 'view', 'type': 'function'}, {
           'inputs': [{'internalType': 'uint32', 'name': '', 'type': 'uint32'},
                      {'internalType': 'bytes32', 'name': '', 'type': 'bytes32'}], 'name': 'nextNonce',
           'outputs': [{'internalType': 'uint64', 'name': 'nonce', 'type': 'uint64'}], 'stateMutability': 'view',
           'type': 'function'}, {'inputs': [], 'name': 'numeratorNative',
                                 'outputs': [{'internalType': 'uint256', 'name': '', 'type': 'uint256'}],
                                 'stateMutability': 'view', 'type': 'function'},
       {'inputs': [], 'name': 'numeratorUsdc', 'outputs': [{'internalType': 'uint256', 'name': '', 'type': 'uint256'}],
        'stateMutability': 'view', 'type': 'function'},
       {'inputs': [], 'name': 'numeratorUsdt', 'outputs': [{'internalType': 'uint256', 'name': '', 'type': 'uint256'}],
        'stateMutability': 'view', 'type': 'function'}, {'inputs': [], 'name': 'oAppVersion', 'outputs': [
        {'internalType': 'uint64', 'name': 'senderVersion', 'type': 'uint64'},
        {'internalType': 'uint64', 'name': 'receiverVersion', 'type': 'uint64'}], 'stateMutability': 'view',
                                                         'type': 'function'},
       {'inputs': [], 'name': 'owner', 'outputs': [{'internalType': 'address', 'name': '', 'type': 'address'}],
        'stateMutability': 'view', 'type': 'function'},
       {'inputs': [{'internalType': 'uint32', 'name': 'eid', 'type': 'uint32'}], 'name': 'peers',
        'outputs': [{'internalType': 'bytes32', 'name': 'peer', 'type': 'bytes32'}], 'stateMutability': 'view',
        'type': 'function'}, {'inputs': [{'internalType': 'uint32', 'name': '_dstEid', 'type': 'uint32'},
                                         {'internalType': 'uint256', 'name': '_zroAmount', 'type': 'uint256'}],
                              'name': 'quoteClaimCallback', 'outputs': [{'components': [
            {'internalType': 'uint256', 'name': 'nativeFee', 'type': 'uint256'},
            {'internalType': 'uint256', 'name': 'lzTokenFee', 'type': 'uint256'}],
            'internalType': 'struct MessagingFee',
            'name': 'msgFee', 'type': 'tuple'}],
                              'stateMutability': 'view', 'type': 'function'},
       {'inputs': [], 'name': 'renounceOwnership', 'outputs': [], 'stateMutability': 'nonpayable', 'type': 'function'},
       {'inputs': [{'internalType': 'uint256', 'name': '_zroAmount', 'type': 'uint256'}], 'name': 'requiredDonation',
        'outputs': [{'components': [{'internalType': 'uint256', 'name': 'usdc', 'type': 'uint256'},
                                    {'internalType': 'uint256', 'name': 'usdt', 'type': 'uint256'},
                                    {'internalType': 'uint256', 'name': 'native', 'type': 'uint256'}],
                     'internalType': 'struct Donation', 'name': '', 'type': 'tuple'}], 'stateMutability': 'view',
        'type': 'function'},
       {'inputs': [{'internalType': 'address', 'name': '_delegate', 'type': 'address'}], 'name': 'setDelegate',
        'outputs': [], 'stateMutability': 'nonpayable', 'type': 'function'},
       {'inputs': [{'internalType': 'address', 'name': '_donateAndClaim', 'type': 'address'}],
        'name': 'setDonateAndClaim', 'outputs': [], 'stateMutability': 'nonpayable', 'type': 'function'},
       {'inputs': [{'internalType': 'bytes32', 'name': '_merkleRoot', 'type': 'bytes32'}], 'name': 'setMerkleRoot',
        'outputs': [], 'stateMutability': 'nonpayable', 'type': 'function'}, {
           'inputs': [{'internalType': 'uint32', 'name': '_eid', 'type': 'uint32'},
                      {'internalType': 'bytes32', 'name': '_peer', 'type': 'bytes32'}], 'name': 'setPeer',
           'outputs': [], 'stateMutability': 'nonpayable', 'type': 'function'},
       {'inputs': [{'internalType': 'address', 'name': 'newOwner', 'type': 'address'}], 'name': 'transferOwnership',
        'outputs': [], 'stateMutability': 'nonpayable', 'type': 'function'}, {
           'inputs': [{'internalType': 'address', 'name': '_to', 'type': 'address'},
                      {'internalType': 'uint256', 'name': '_amount', 'type': 'uint256'}], 'name': 'withdrawZro',
           'outputs': [], 'stateMutability': 'nonpayable', 'type': 'function'},
       {'inputs': [{'internalType': 'address', 'name': 'user', 'type': 'address'}], 'name': 'zroClaimed',
        'outputs': [{'internalType': 'uint256', 'name': 'amount', 'type': 'uint256'}], 'stateMutability': 'view',
        'type': 'function'},
       {'inputs': [], 'name': 'zroToken', 'outputs': [{'internalType': 'address', 'name': '', 'type': 'address'}],
        'stateMutability': 'view', 'type': 'function'}]


class L0Claim(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def donateAndClaim(self, account, donate_amount, amount_to_claim, merkle_proof):
        txn_params = self.build_generic_data(account.address, False)
        txn_params['value'] = donate_amount

        return self.contract.functions.donateAndClaim(
            2,
            donate_amount,
            #5520000000000000
            #5520000000000000
            amount_to_claim,
            #183461138113000003000
            #183461138113000003000
            merkle_proof,
            #['0x1fce26056b7cc81752ae974a8baa89d78123fd539f5fb47f0e5cb1f2c603b3c8', '0x63e5fe91dbafe187829bc84f8e6ddb6c6f4d62e5ef1ff4796913101fc0ea916e', '0x9a734b2f26f5334741ee26eda57cef6c4f84430e1d91be1b65cb4e7be577d0e9', '0xcc8bc1df42652bfd0aa8032a1cf50a31e31ecbe78d52411491076bfc59b70e7d', '0x8edc7e63ae09f38a3e00b6ee33fb102eee1771ce73b8a6b089cfcc3c2b4b7b8f', '0x8870668769d7dfa64e8695bf51741c73358923b7ad7e07dd6055c088760063cc', '0x75321dbf87f550059a302d401fd3aae9b13204316e202b3d0bd459447d611739', '0x5d48d0538089ccbfade4265ff10efae09691a0204dcef8db5b20064d3c6103bb', '0xa6d7408758dc62d68fc1d768714f4f9422bb1ad8f222a687c0e1c4d88e7ddfcf', '0xfa0a2c7465fccd5838f147abad0b3f928b716eabfbbae622de7cfdb0c0fcbce1', '0x30369b62a912a4555e898133ad536bbc8b58af456cdb1001e38a8e0e4d01c034', '0xea79cef9445e78b7c5695d0bb5e5d676bfa5b7ced66df915ec13fb6fd6b3e131', '0xc4abfec16adca47f0c0d5d22dcb87586b66cba0048a684e440be04175dbc5c58', '0xecffcc8af84ed13272ad00aeee065c02e4778b6cfd0d124222f23264b0679076', '0x767c848c0da4bee189851e644bb437e9ef426d6e418f14fd3e9337e5ffe3596f', '0x55ab35e3f06e43b8af688169490819fdcb4180258210acb11db1976379754b1c', '0x950b026464be9f6fd40516d45236a70f07ff78b3dc1b9d258c9ddbf7fd0a003f', '0x6b71fb967b62b9e1f399a856517a1da9c22a0993fdb84b1b998b14849c2d7e9e', '0x9ae69f117d127a35e00c470d6bff380bfabbca78d86668984e867861853ee8fc', '0xa042d488b6a933c88ec4b41ba407ff13521c2e67e842697017bc19ed9f9a8e5d', '0x65b28f8ec7c9c9fc471c6a5000a1906b04e8da896a739ffddf6d3b71e47fd60d']
            #['0x1fce26056b7cc81752ae974a8baa89d78123fd539f5fb47f0e5cb1f2c603b3c8', '0x63e5fe91dbafe187829bc84f8e6ddb6c6f4d62e5ef1ff4796913101fc0ea916e', '0x9a734b2f26f5334741ee26eda57cef6c4f84430e1d91be1b65cb4e7be577d0e9', '0xcc8bc1df42652bfd0aa8032a1cf50a31e31ecbe78d52411491076bfc59b70e7d', '0x8edc7e63ae09f38a3e00b6ee33fb102eee1771ce73b8a6b089cfcc3c2b4b7b8f', '0x8870668769d7dfa64e8695bf51741c73358923b7ad7e07dd6055c088760063cc', '0x75321dbf87f550059a302d401fd3aae9b13204316e202b3d0bd459447d611739', '0x5d48d0538089ccbfade4265ff10efae09691a0204dcef8db5b20064d3c6103bb', '0xa6d7408758dc62d68fc1d768714f4f9422bb1ad8f222a687c0e1c4d88e7ddfcf', '0xfa0a2c7465fccd5838f147abad0b3f928b716eabfbbae622de7cfdb0c0fcbce1', '0x30369b62a912a4555e898133ad536bbc8b58af456cdb1001e38a8e0e4d01c034', '0xea79cef9445e78b7c5695d0bb5e5d676bfa5b7ced66df915ec13fb6fd6b3e131', '0xc4abfec16adca47f0c0d5d22dcb87586b66cba0048a684e440be04175dbc5c58', '0xecffcc8af84ed13272ad00aeee065c02e4778b6cfd0d124222f23264b0679076', '0x767c848c0da4bee189851e644bb437e9ef426d6e418f14fd3e9337e5ffe3596f', '0x55ab35e3f06e43b8af688169490819fdcb4180258210acb11db1976379754b1c', '0x950b026464be9f6fd40516d45236a70f07ff78b3dc1b9d258c9ddbf7fd0a003f', '0x6b71fb967b62b9e1f399a856517a1da9c22a0993fdb84b1b998b14849c2d7e9e', '0x9ae69f117d127a35e00c470d6bff380bfabbca78d86668984e867861853ee8fc', '0xa042d488b6a933c88ec4b41ba407ff13521c2e67e842697017bc19ed9f9a8e5d', '0x65b28f8ec7c9c9fc471c6a5000a1906b04e8da896a739ffddf6d3b71e47fd60d']
            account.address,
            '0x'
        ).build_transaction(txn_params)



import random
from datetime import datetime

from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/nile_pool.json")


class NileContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def add_liquidity_eth(self, account, token, max_token, amount_eth):
        tx_data = self.build_generic_data(account.address, False)

        deadline = int(datetime.now().timestamp()) + random.randint(60, 60 * 60 * 24)
        tx_data['value'] = amount_eth.wei

        return self.contract.functions.addLiquidityETH(
            token,
            False,
            int(max_token * 0.99),
            int(max_token * 0.95),
            int(amount_eth.wei * 0.95),
            account.address,
            deadline
        ).build_transaction(tx_data)

    @evm_transaction
    def remove_liquidity_eth(self, account, token, liquidity, amount_eth, max_token):
        tx_data = self.build_generic_data(
            account.address,
            False
        )

        deadline = int(datetime.now().timestamp()) + random.randint(60, 60 * 60 * 24)

        return self.contract.functions.removeLiquidityETH(
            token,
            False,
            int(liquidity * 0.99),
            int(max_token * 0.95),
            int(amount_eth * 0.95),
            account.address,
            deadline
        ).build_transaction(tx_data)

    def quote_remove_liquidity_eth(self, token_in, token_out, liquidity):
        return self.contract.functions.quoteRemoveLiquidity(token_in, token_out, False, liquidity).call()

    def get_amount_out(self, amount, from_token, to_token):
        return self.contract.functions.getAmountOut(
            amount.wei,
            Web3.to_checksum_address(from_token),
            Web3.to_checksum_address(to_token)
        ).call()



import requests
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.domain.dex import Dex
from sybil_engine.utils.gas_utils import l1_gas_price
from sybil_engine.utils.utils import randomized_sleeping, SwapException
from web3 import Web3


class OneInch(Dex):
    dex_name = '1inch'
    swap_contract = '1INCH'
    supported_chains = ['BASE', 'ZKSYNC', 'OPTIMISM', 'ARBITRUM']

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.weth_address = self.tokens['WETH']
        self.base_url = 'https://api.1inch.dev/swap'
        self.inch_version = 5.2

    @l1_gas_price
    def swap(self, amount_to_swap, from_token, to_token, slippage, account):
        if amount_to_swap.wei == 0:
            raise SwapException(f"Can't swap 0 of {amount_to_swap.token}")

        self.get_txn(account, amount_to_swap, from_token, to_token)

    def get_api_call_data(self, url):
        response = requests.get(url, headers={'Authorization': "DuagMR3vHpYc9JBNqM0Wx9mSWA7VzwuH"})

        return response.json()

    @evm_transaction
    def get_txn(self, account, amount_to_swap, from_token, to_token):
        spender_json = self.get_api_call_data(
            f'{self.base_url}/v{self.inch_version}/{self.web3.eth.chain_id}/approve/spender')
        spender = Web3.to_checksum_address(spender_json['address'])

        if from_token.symbol() != 'ETH' and from_token.allowance(account, spender) < amount_to_swap.wei:
            from_token.approve(account, spender)

        randomized_sleeping({'from': 1, 'to': 5})

        _1inchurl = f"{self.base_url}/v{self.inch_version}/{self.web3.eth.chain_id}/swap?fromTokenAddress={from_token.address()}&toTokenAddress={to_token.address()}&amount={amount_to_swap.wei}&fromAddress={account.address}&slippage=5"
        json_data = self.get_api_call_data(_1inchurl)

        if json_data is False:
            raise Exception(json_data['description'])

        contract_txn = json_data['tx']
        contract_txn['from'] = Web3.to_checksum_address(contract_txn['from'])
        contract_txn['chainId'] = self.web3.eth.chain_id
        contract_txn['nonce'] = self.web3.eth.get_transaction_count(account.address)
        contract_txn['to'] = Web3.to_checksum_address(contract_txn['to'])
        contract_txn['gasPrice'] = int(contract_txn['gasPrice'])
        contract_txn['gas'] = int(contract_txn['gas'])
        contract_txn['value'] = int(contract_txn['value'])

        return contract_txn



from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3
from web3 import Web3

from web3_wizzard_lib.core.contract.octomos import OctomosContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class Octomos(SubModule):
    module_name = 'OCTOMOS'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['OCTOMOS']

        octomos_contract = OctomosContract(contract_address, web3)

        octomos_contract.launchpadBuy(
            account,
            Web3.to_bytes(hexstr='0x0c21cfbb').ljust(4, b'\0'),
            Web3.to_bytes(hexstr='0x53b93973').ljust(4, b'\0'),
            0,
            1,
            [],
            b''
        )

    def log(self):
        return "OCTOMOS"



from sybil_engine.module.module import Module
from sybil_engine.utils.utils import ConfigurationException
from sybil_engine.utils.validation_utils import validate_amount_interval, validate_chain

from web3_wizzard_lib.core.modules.bridge.coredao_bridge_sub import CoreDaoBridgeSubModule
from web3_wizzard_lib.core.modules.bridge.orbiter_sub import OrbiterSubModule
from web3_wizzard_lib.core.modules.bridge.scroll_bridge_sub import ScrollBridgeSubModule
from web3_wizzard_lib.core.modules.bridge.stargate_v2_sub import StargateV2SubModule


class BridgeModule(Module):
    module_name = 'BRIDGE'
    module_config = 'bridge_module'

    supported_tokens = ['NATIVE', 'USDC', 'USDT', 'WETH']

    def execute(self, bridge, bridge_amount_interval, from_chain, to_chain, token, account):
        if bridge == 'ORBITER':
            module = OrbiterSubModule()
        elif bridge == 'STARGATE_V2':
            module = StargateV2SubModule()
        elif bridge == 'COREDAO':
            module = CoreDaoBridgeSubModule()
        elif bridge == 'SCROLL':
            module = ScrollBridgeSubModule()
        else:
            raise ConfigurationException(f'Bridge {bridge} is not supported')

        module.execute(bridge_amount_interval, from_chain, to_chain, token, self.min_native_balance, account)

    def log(self):
        return "BRIDGE"

    def parse_params(self, module_params):
        validate_amount_interval(module_params['bridge_amount_interval'])
        validate_chain(module_params['from_chain'])

        if 'token' not in module_params:
            module_params['token'] = 'NATIVE'

        if module_params['token'] not in self.supported_tokens:
            raise ConfigurationException(
                f"{module_params['token']} not supported in orbiter. Supported options: {self.supported_tokens}")

        return module_params['bridge'], module_params['bridge_amount_interval'], module_params['from_chain'], \
            module_params['to_chain'], \
            module_params['token']



import functools

from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.domain.balance.balance import NativeBalance, NotEnoughNativeBalance
from sybil_engine.domain.balance.balance_utils import get_native_balance, from_eth_to_wei
from sybil_engine.utils.l0_utils import L0FeeToHigh
from sybil_engine.utils.utils import randomized_sleeping


def stargate_check_gas(token):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(self, *args):
            check_gas(token, self.chain_instance, args[-1], args[0], self.web3)
            return func(self, *args)

        return wrapper

    return decorator


def check_gas(token, chain_instance, to_chain_instance, account, web3):
    while True:
        try:
            verify_stargate_fee(token, chain_instance, to_chain_instance, account, web3)
            return
        except L0FeeToHigh as e:
            logger.info(e)
            randomized_sleeping({'from': 60 * 4, 'to': 60 * 8})
        except Exception as e:
            raise StargateBridgeException("Stargate bridge error") from e


def verify_stargate_fee(token, from_chain_instance, to_chain_instance, account, web3):
    native_balance = get_native_balance(account, web3, from_chain_instance)
    native_fee_balance = get_native_fee_balance(token, native_balance, from_chain_instance, to_chain_instance, account,
                                                web3)

    logger.info(f"Native LayerZero fee: {native_fee_balance.log_line()}")
    if native_fee_balance.wei > native_balance.wei:
        raise NotEnoughNativeBalance(
            f"Native balance ({native_balance.log_line()}) < Native LayerZero required fee")
    if native_fee_balance.wei > from_eth_to_wei(from_chain_instance['max_l0_fee']):
        raise L0FeeToHigh(f"Native LayerZero fee is to high")


def get_native_fee_balance(token, native_balance, from_chain_instance, to_chain_instance, account, web3):
    chain_contracts = get_contracts_for_chain(from_chain_instance['chain'])

    if token in ['ETH', 'USDC']:
        from web3_wizzard_lib.core.contract.stargate_router import StargateRouter
        stargate_router = StargateRouter(chain_contracts["STARGATE_ROUTER"], web3)

        wei_balance = stargate_router.count_native_fee_stargate(
            to_chain_instance['stargate_chain_id'],
            account.address
        )
    else:
        raise Exception("Token not supported")

    return NativeBalance(wei_balance, native_balance.chain, native_balance.token)


class StargateBridgeException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)



from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.domain.dex import Dex

from web3_wizzard_lib.core.contract.maverick_multicall import MaverickMulticall
from ...contract.pancake_pool import PancakePool
from ...contract.pancake_quoter import PancakeQuoter

pool_addresses = {
    'ZKSYNC': {
        "ETH>USDC": "0x41c8cf74c27554a8972d3bf3d2bd4a14d8b604ab"
    },
    'BASE': {
        "ETH>USDbC": "0x06e6736ca9e922766279a22b75a600fe8b8473b6",
        "ETH>USDC": "0x06e6736ca9e922766279a22b75a600fe8b8473b6"
    }
}


class Maverick(Dex):
    dex_name = 'maverick'
    swap_contract = 'MAV_ROUTER'
    supported_chains = ['ZKSYNC', 'BASE']

    def __init__(self, chain_instance, web3):
        self.chain_instance = chain_instance
        super().__init__(chain_instance, web3)
        self.weth_contract = self.tokens['WETH']
        self.pancake_pool = PancakePool(self.chain_contracts["PANCAKE_FACTORY"], self.web3)
        self.maverick_router = MaverickMulticall(self.chain_contracts[self.swap_contract], self.web3)

    def swap_native_for_token(self, account, amount_to_swap, slippage, to_token_address):
        erc20_token = Erc20Token(self.chain_instance['chain'], to_token_address, self.web3)
        pair_name = f"{self.chain_instance['gas_token']}>{erc20_token.symbol()}"
        maverick_pool_address = pool_addresses[self.chain_instance['chain']][pair_name]

        pool = self.pancake_pool.get_pool(self.weth_contract, to_token_address)

        if pool == self.tokens['ZERO_ADDRESS']:
            raise Exception(f"[{account.address}] Swap path {self.weth_contract} to {to_token_address} not found!")

        amount_out = int(self.get_amount_out_min(amount_to_swap, self.weth_contract, to_token_address) * slippage)

        args = [account, amount_to_swap, amount_out, self.weth_contract, maverick_pool_address, to_token_address]

        return args, self.maverick_router.multicall

    def swap_token_for_native(self, account, amount_to_swap, from_token_address, slippage):
        pair_name = f'{self.chain_instance["gas_token"]}>{amount_to_swap.token}'
        maverick_pool_address = pool_addresses[self.chain_instance['chain']][pair_name]

        pool = self.pancake_pool.get_pool(from_token_address, self.weth_contract)

        if pool == self.tokens['ZERO_ADDRESS']:
            raise Exception(f"[{account.address}] Swap path {from_token_address} to {self.weth_contract} not found!")

        amount_out = int(self.get_amount_out_min(amount_to_swap, from_token_address, self.weth_contract) * slippage)

        args = [account, amount_to_swap, amount_out, from_token_address, maverick_pool_address, self.weth_contract]

        return args, self.maverick_router.multicall

    def get_amount_out_min(self, amount_to_swap, token_in_address, token_out_address):
        pancake_quoter_contract = self.chain_contracts["PANCAKE_QUOTER"]
        pancake_quoter = PancakeQuoter(pancake_quoter_contract, self.web3)

        return pancake_quoter.quote_exact_input_single(token_in_address, token_out_address, amount_to_swap.wei)



from sybil_engine.contract.send import Send
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.utils.app_account_utils import AppAccount

from web3_wizzard_lib.core.modules.nft.nft_submodule import NftSubmodule


class LineaCultureWeek2Day2(NftSubmodule):
    module_name = 'LINEA_CULTURE_2_4'
    nft_address = '0x32DeC694570ce8EE6AcA08598DaEeA7A3e0168A3'

    def execute(self, account: AppAccount, chain='LINEA', wei_cost=0):
        Send(
            None,
            self.create_web3(account, chain)
        ).send_to_wallet(
            account,
            self.nft_address,
            NativeBalance(wei_cost, chain, "ETH"),
            "0x00000000"
        )

    def log(self):
        return "LINEA CULTURE 2 WEEK DAY 4 (FROGWAR)"



import requests
from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.module.module import Module
from web3 import Web3

from web3_wizzard_lib.core.utils.statistic_utils import get_statistic_writer, statistic_date_string


class AirdropPrinter(Module):
    module_name = 'AIRDROP_PRINTER'
    module_config = None

    claimers = {
        "L0": "0xB09F16F625B363875e39ADa56C03682088471523",
        "SCROLL": "0xE8bE8eB940c0ca3BD19D911CD3bEBc97Bea0ED62",
        "ZKSYNC": "0x66Fd4FC8FA52c9bec2AbA368047A0b27e24ecfe4",
        "ORBITER": "0x13dFDd3a9B39323F228Daf73B62C23F7017E4679",
        "ODOS": "0x4C8f8055D88705f52c9994969DDe61AB574895a3"
    }

    EMPTY_ADDRESS = '-'

    chains = ["ARBITRUM", "ZKSYNC", "SCROLL", "BASE"]

    def execute(self, account, statistic_write='GOOGLE'):
        chain_contracts = {}
        for chain in self.chains:
            chain_instance = get_chain_instance(chain)

            datas = self.find_interacted_contracts_hash(
                account.address,
                chain_instance['api_scan'],
                chain_instance['api_scan_key']
            )

            chain_contracts[chain] = datas['result']

        account_links = {}

        for project, claim_address in self.claimers.items():
            for chain, datas in chain_contracts.items():
                chain_instance = get_chain_instance(chain)
                for data in datas:
                    if data['to'] != "" and claim_address == Web3.to_checksum_address(data['to']):
                        account_links[project] = chain_instance['scan'] + "tx/" + data['hash']

            if account_links.get(project) is None:
                account_links[project] = self.EMPTY_ADDRESS

        logger.info(account_links)
        statistics_writer = get_statistic_writer()
        job_name = f"airdrops_{statistic_date_string}"
        statistics_writer.init_if_required(job_name, ['#', 'Address'] + list(self.claimers.keys()))

        value_row = [account.app_id, account.address]
        for project, address in account_links.items():
            value_row.append(address)

        statistics_writer.write_row(job_name, value_row)

    def find_interacted_contracts_hash(self, wallet_address, api_url, api_key):
        api_url = f"{api_url}/api?module=account&action=txlist&address={wallet_address}&startblock=0&endblock=latest&sort=asc&apikey={api_key}&offset=200"

        if "base" in api_url:
            logger.info(f"BASE {api_url}")
            pass

        response = requests.get(api_url)
        data = response.json()

        if data['status'] == '1' and data['message'] == 'OK':
            return data
        else:
            logger.info("nothing")
            logger.info(data)
            pass
            return data

    def log(self):
        return "AIRDROP PRINTER"



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction

from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/eralend.json")


class EraLendContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def mint(self, account, amount: int):
        sender = account.address

        txn_params = self.build_generic_data(sender)
        txn_params['data'] = 0x1249c58b
        txn_params['gas'] = int(self.web3.eth.estimate_gas(txn_params))

        txn_params['value'] = amount

        return txn_params

    @evm_transaction
    def redeem_underlying(self, account, amount: int):
        sender = account.address

        txn_params = self.build_generic_data(sender, False)

        return self.contract.functions.redeemUnderlying(amount).build_transaction(txn_params)

    def get_deposit_amount(self, account):
        return self.contract.functions.balanceOfUnderlying(account.address).call()



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/micro3.json")


class Micro3Contract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def purchase(self, account, amount):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = 80000000000000

        return self.contract.functions.purchase(amount).build_transaction(txn_params)



import requests
from sybil_engine.config.app_config import get_network

from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi
from sybil_engine.utils.utils import SwapException

abi = load_abi("resources/abi/lineaswap_router.json")

referral = {
    324: 387663447,
    260: 387663447,
    8453: 1030612943,
    18453: 1030612943,
    59144: 268273819
}

chain_map_id = {
    'BASE': 8453,
    'ZKSYNC': 324
}


class Odos(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def quote(self, account, from_token: str, to_token: str, amount: int, slippage: float, chain: str):
        chain_id = self.get_chain_id(chain)

        url = "https://api.odos.xyz/sor/quote/v2"

        data = {
            "chainId": chain_id,
            "inputTokens": [
                {
                    "tokenAddress": from_token,
                    "amount": f"{amount}"
                }
            ],
            "outputTokens": [
                {
                    "tokenAddress": to_token,
                    "proportion": 1
                }
            ],
            "slippageLimitPercent": slippage,
            "userAddr": account.address,
            "referralCode": referral[self.web3.eth.chain_id] if True is True else 0,
            "compact": True
        }

        response = self.proxy_request(account, data, url)

        if response.status_code == 200:
            return response.json()
        else:
            raise SwapException(f"[{account.app_id}][{account.address}] Bad Odos request")

    def assemble(self, account, path_id):
        url = "https://api.odos.xyz/sor/assemble"

        data = {
            "userAddr": account.address,
            "pathId": path_id,
            "simulate": False,
        }

        response = self.proxy_request(account, data, url)

        if response.status_code == 200:
            return response.json()
        else:
            raise SwapException(f"[{account.app_id}][{account.address}] Bad Odos request")

    def proxy_request(self, account, data, url):
        proxies = {}
        proxies.update(
            {"http": account.proxy}
        )
        response = requests.post(
            url=url,
            headers={"Content-Type": "application/json"},
            json=data,
            proxies=proxies
        )
        return response

    @evm_transaction
    def swap(self, account, transaction_data):
        transaction = transaction_data["transaction"]

        transaction["nonce"] = self.web3.eth.get_transaction_count(transaction['from'])
        transaction["chainId"] = self.web3.eth.chain_id
        transaction["value"] = int(transaction["value"])

        return transaction

    def get_chain_id(self, chain):
        if get_network() == 'MAIN':
            return self.web3.eth.chain_id
        else:
            return chain_map_id[chain]



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_starknet_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/orbiter_starknet.json")


class OrbiterStarknet(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_starknet_transaction
    def bridge(self, account, amount):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = amount.wei

        if account.starknet_address[:3] == '0x0':
            starknet_wallet = f'030{account.starknet_address[3:]}'
        else:
            starknet_wallet = f'030{account.starknet_address[2:]}'

        starknet_wallet = bytes.fromhex(starknet_wallet)

        contract_txn = self.contract.functions.transfer(
            '0x80C67432656d59144cEFf962E8fAF8926599bCF8',
            starknet_wallet
        ).build_transaction(txn_params)

        return contract_txn



import requests
from sybil_engine.contract.send import Send
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei
from sybil_engine.utils.app_account_utils import AppAccount
from sybil_engine.utils.utils import AppException

from web3_wizzard_lib.core.modules.nft.nft_submodule import NftSubmodule


class LineaCultureWeek4Day6(NftSubmodule):
    module_name = 'LINEA_CULTURE_4_6'
    nft_address = '0x8975e0635586C6754C8D549Db0e3C7Ee807D9C8C'

    def execute(self, account: AppAccount, chain='LINEA', wei_cost=from_eth_to_wei(0)):
        url = 'https://public-api.phosphor.xyz/v1/purchase-intents'

        data = {
            "buyer": {
                "eth_address": account.address
            },
            "listing_id": "86a8741b-28dd-42ca-9f2f-dfb173a62099",
            "provider": "MINT_VOUCHER",
            "quantity": 1
        }

        proxy = {
            'http': account.proxy,
            'https': account.proxy
        }

        response = requests.post(url, json=data, proxies=proxy)

        if 'error' in response.json():
            raise AppException(response.json())

        signature = response.json()['data']['signature'][2:]
        expiry = response.json()['data']['voucher']['expiry']
        nonce = response.json()['data']['voucher']['nonce']
        tokenID = response.json()['data']['voucher']['token_id']

        data = f"0xd4dfd6bc0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001000000000000000000000000000000000000000000000000000000000000000{nonce}00000000000000000000000000000000000000000000000000000000{decimal_to_hex(expiry)}0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000{tokenID}000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001400000000000000000000000000000000000000000000000000000000000000041{signature}00000000000000000000000000000000000000000000000000000000000000"

        Send(
            None,
            self.create_web3(account, chain)
        ).send_to_wallet(
            account,
            self.nft_address,
            NativeBalance(wei_cost, chain, "ETH"),
            data
        )

    def log(self):
        return "LINEA CULTURE 4 WEEK DAY 6 (Laurent)"


def decimal_to_hex(decimal):
    if decimal == 0:
        return "0x0"
    hex_digits = "0123456789ABCDEF"
    hex_result = ""
    while decimal > 0:
        remainder = decimal % 16
        hex_result = hex_digits[remainder] + hex_result
        decimal = decimal // 16
    return hex_result.lower()



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/linea_day_3.json")


class LineaDay3(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    # @evm_transaction
    # def mintWithVoucher(self, account):
    #     txn_params = self.build_generic_data(account.address, False)
    #
    #     contract_txn = self.contract.functions.mintWithVoucher(
    #         (
    #             '0x0000000000000000000000000000000000000000'
    #             '0x0000000000000000000000000000000000000000',
    #             0,
    #             1,
    #             1,
    #             int(datetime.now().timestamp() + 60 * 10),
    #             0,
    #             1,
    #             '0x0000000000000000000000000000000000000000'
    #         ),
    #         '0xbb6fef09ef7f8958207a489df32fa334879b963aa975b8a62a3839e3c887d5540f05bdeba16649c507c8350a987a2556425707b6f01f4592b744d9767c0b47ca1c'
    #     ).build_transaction(txn_params)
    #
    #     return contract_txn

    @evm_transaction
    def claim(self, account):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = from_eth_to_wei(0.0000029)

        contract_txn = self.contract.functions.claim(
            account.address,  # address
            0,  # uint256
            1,  # uint256
            '0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE',  # address
            0,  # uint256
            (
                [],  # bytes32[] - This needs to be populated with actual data or kept as empty if applicable
                2,  # uint256
                0,  # uint256
                '0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE'  # address
            ),
            '0x'  # bytes - This typically represents an empty byte string or could be filled if needed
        ).build_transaction(txn_params)

        return contract_txn



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/horizondex_quoter.json")


class HorizonDexQuoter(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def quote_exact_input_single(self, amount: int, from_token_address: str, to_token_address: str):
        return self.contract.functions.quoteExactInputSingle(
            (from_token_address, to_token_address, amount, 40, 0)
        ).call()[1]



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/abbysworld.json")


class ReadOnContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def curate(self, account, contentUrl):
        txn_params = self.build_generic_data(account.address, True)

        txn_params['data'] = f'0x7859bb8d{contentUrl}'
        txn_params['gasPrice'] = self.web3.eth.gas_price

        return txn_params



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/woofi.json")


class ZKDXContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)
        self.zkdx_token = '176211869ca2b568f2a7d4ee941e073a821ee1ff'

    @evm_transaction
    def swap_to_zkdx(self, account, amount):
        sender = account.address
        txn_params = self.build_generic_data(sender, True)

        amount_hex = self.decimal_to_hex_padded(amount.wei)
        txn_params[
            'data'] = f'0x045d0389000000000000000000000000{self.zkdx_token}00000000000000000000000000000000{amount_hex}'

        return txn_params

    @evm_transaction
    def swap_from_zkdx(self, account, amount):
        sender = account.address
        txn_params = self.build_generic_data(sender, True)

        amount_hex = self.decimal_to_hex_padded(amount)

        # txn_params['gas'] = 60000

        txn_params[
            'data'] = f'0x1e9a6950000000000000000000000000{self.zkdx_token}00000000000000000000000000000000{amount_hex}'

        return txn_params

    def decimal_to_hex_padded(self, decimal_number):
        hex_number = hex(decimal_number)[2:]  # Remove the '0x' prefix
        return hex_number.zfill(32)  # Pad with zeros to make the length 32



from inspect import getsourcefile
from random import randint
from requests import post
from loguru import logger
from time import sleep
from tqdm import tqdm
import sys
import ctypes
import os
sys.__stdout__ = sys.stdout

TG_BOT_TOKEN = '' # токен от тг бота (`12345:Abcde`) для уведомлений. если не нужно - оставляй пустым
TG_USER_ID = [] # тг айди куда должны приходить уведомления. [21957123] - для отправления уведомления только себе, [21957123, 103514123] - отправлять нескольким людями

windll = ctypes.windll if os.name == 'nt' else None # for Mac users


class WindowName:
    def __init__(self):
        try: self.path = os.path.abspath(getsourcefile(lambda: 0)).split("\\")[-2]
        except: self.path = os.path.abspath(getsourcefile(lambda: 0)).split("/")[-2]

        self.accs_done = 0
        self.modules_amount = 0
        self.modules_done = 0

    def update_accs(self):
        self.accs_done += 1
        self.modules_amount = 0
        self.modules_done = 0

    def update_modules(self):
        self.modules_done += 1

    def set_modules(self, modules_amount: int):
        self.modules_amount = modules_amount


class TgReport:
    def __init__(self):
        self.logs = ''


    def update_logs(self, text: str):
        self.logs += f'{text}\n'


    def send_log(self, wallet, window_name, mode: str):
        notification_text = f'[{window_name.accs_done}/{window_name.accs_amount}] <i>{wallet.address}</i>\n\n' \
                            f'{self.logs}\n'
        if mode == 'Claim quests': notification_text += f'{wallet.stats.get("status")}'

        texts = []
        while len(notification_text) > 0:
            texts.append(notification_text[:1900])
            notification_text = notification_text[1900:]

        if TG_BOT_TOKEN:
            for tg_id in TG_USER_ID:
                for text in texts:
                    try: post(f'https://api.telegram.org/bot{TG_BOT_TOKEN}/sendMessage?parse_mode=html&chat_id={tg_id}&text={text}')
                    except Exception as err: logger.error(f'[-] TG | Send Telegram message error to {tg_id}: {err}')


def sleeping(*timing):
    if type(timing[0]) == list: timing = timing[0]
    if len(timing) == 2: x = randint(timing[0], timing[1])
    else: x = timing[0]
    for _ in tqdm(range(x), desc='sleep ', bar_format='{desc}: {n_fmt}/{total_fmt}'):
        sleep(1)


def make_text_border(text: str):
    new_text = ''
    max_len = max([len(string) for string in text.split('\n')])

    new_text += '+' + '-' * (max_len + 8) + '+\n'
    for string in text.split('\n'): new_text += f'\t{string}\n'
    new_text += '+' + '-' * (max_len + 8) + '+\n'

    return new_text



from urllib.parse import urlparse

from loguru import logger
from sybil_engine.module.module import Module
import requests


class AdsImportProxy(Module):
    module_name = 'ADS_IMPORT_PROXY'
    module_config = 'ads_import_config'

    def execute(self, ads_url, account):
        parsed_proxy = urlparse(account.proxy)
        proxy_type = parsed_proxy.scheme
        proxy_host = parsed_proxy.hostname
        proxy_port = parsed_proxy.port
        proxy_user = parsed_proxy.username
        proxy_password = parsed_proxy.password

        logger.info(f"Import proxy {account.proxy} for account {account.app_id}")

        response = requests.get(f"{ads_url}/list", params={"serial_number": str(account.app_id)})
        user_id = response.json()['data']['list'][0]["user_id"]

        self.updateProfile(ads_url, user_id, proxy_host, proxy_password, proxy_port, proxy_type, proxy_user)

    def updateProfile(self, ads_url, user_id, proxy_host, proxy_password, proxy_port, proxy_type, proxy_user):
        data = {
            "user_id": user_id,
            "user_proxy_config": {
                "proxy_type": proxy_type,
                "proxy_host": proxy_host,
                "proxy_port": str(proxy_port),
                "proxy_user": proxy_user,
                "proxy_password": proxy_password,
                "proxy_soft": "luminati"
            }
        }
        response = requests.post(f"{ads_url}/update", json=data)
        logger.info(response.status_code)
        logger.info(response.json())

    def log(self):
        return "ADS IMPORT PROXY"

    def parse_params(self, module_params):
        if 'ads_url' not in module_params:
            module_params['ads_url'] = "http://local.adspower.net:50325/api/v1/user"

        return [
            module_params['ads_url']
        ]



from sybil_engine.contract.contract import Contract
from sybil_engine.data.networks import get_ids_chain
from sybil_engine.data.tokens import get_tokens_for_chain

abi = 'test'


class MendiFinanceContract(Contract):
    def __init__(self, contract_address, web3):
        self.weth_token = get_tokens_for_chain(get_ids_chain()[web3.eth.chain_id])['WETH']
        super().__init__(contract_address, web3, abi)



from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.domain.balance.balance import Erc20Balance, NativeBalance

from web3_wizzard_lib.core.contract.stargate_token_pool import StargateTokenPool


class StargatePoolToken:
    def __init__(self, chain, token, web3):
        self.chain = chain
        self.token = token
        self.web3 = web3
        self.token_pool_contract = StargateTokenPool(get_tokens_for_chain(self.chain)[self.token], self.web3)

    def balance(self, account):
        if self.token == 'STARGATE_USDC_POOL':
            return Erc20Balance(self.token_pool_contract.balance_of(account), self.chain, self.token)
        else:
            return NativeBalance(self.token_pool_contract.balance_of(account), self.chain, self.token)

    def approve(self, account, contract_on_approve):
        return self.token_pool_contract.approve(account, contract_on_approve)

    def allowance(self, account, allowance_contract):
        return self.token_pool_contract.allowance(account, allowance_contract)




from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction, l0_evm_transaction
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/merkly.json")


class MerklyMinter(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def mint(self, account):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = Web3.to_wei(0.0004, "ether")
        contract_txn = self.contract.functions.mint().build_transaction(txn_params)

        return contract_txn

    @l0_evm_transaction
    def refuel(self, account, to_chain_id, amount_wei, fee):
        txn_params = self.build_generic_data(account.address, False)

        hex_amount = self.decimal_to_hex_padded(amount_wei)

        txn_params['value'] = amount_wei + fee
        contract_txn = self.contract.functions.bridgeGas(
            to_chain_id,
            account.address,
            f'0x00020000000000000000000000000000000000000000000000000000000000030d4000000000000000000000000000000000{hex_amount}{account.address[2:]}'
        ).build_transaction(txn_params)

        return contract_txn

    def estimate_send_fee(self, account, to_chain_id, hex_amount):
        return self.contract.functions.estimateSendFee(
            to_chain_id,
            b'0x',  # _zroPaymentAddress
            f'0x00020000000000000000000000000000000000000000000000000000000000030d4000000000000000000000000000000000{hex_amount}{account.address[2:]}'
            # _adapterParams
        ).call()[0]

    def decimal_to_hex_padded(self, decimal_number):
        hex_number = hex(decimal_number)[2:]  # Remove the '0x' prefix
        return hex_number.zfill(32)  # Pad with zeros to make the length 32



import requests
from sybil_engine.contract.send import Send
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei
from sybil_engine.utils.app_account_utils import AppAccount

from web3_wizzard_lib.core.modules.nft.nft_submodule import NftSubmodule


class ScrollCanvas1YearBadge(NftSubmodule):
    module_name = 'SCROLL_CANVAS_YEAR_BADGE'
    nft_address = '0x3dacAd961e5e2de850F5E027c70b56b5Afa5DfeD'

    def execute(self, account: AppAccount, chain='SCROLL', wei_cost=from_eth_to_wei(0)):
        url = f"https://canvas.scroll.cat/badge/claim?badge={self.nft_address}&recipient={account.address}"

        proxy = {
            'http': account.proxy,
            'https': account.proxy
        }

        result = requests.get(url, proxies=proxy).json()

        Send(
            None,
            self.create_web3(account, chain)
        ).send_to_wallet(
            account,
            result['tx']['to'],
            NativeBalance(wei_cost, chain, "ETH"),
            result['tx']['data']
        )

    def log(self):
        return "SCROLL CANVAS (1 YEAR BADGE)"



sleep_interval = {'from': 60 * 5 * 1, 'to': 60 * 10 * 1}  # Sleep interval between accounts
swap_retry_sleep_interval = {'from': 60 * 5 * 1, 'to': 60 * 10 * 1}  # Sleep interval between retries

min_native_interval = {'from': 0.5, 'to': 0.5}

module = "STARGATE_AUTO_BRIDGE" # [Название модуля или сценария]
# Modules: [WARMUP, SELL_ALL, CONCRETE_SWAP, WRAPPING, DMAIL, SEND_TO_CEX, ORBITER,
# BUNGEE, STARGATE_CLASSIC_BRIDGE, STARGATE_AUTO_BRIDGE, STARGATE_FARMING, LENDING, RAGE_WITHDRAW,
# LayerBank, LIQUIDITY_POOL, ZKDX, CEX_SUB_ACCOUNT_TRANSFER, SLEEP_MODULE, MERKLY_REFUEL


# Module [WARMUP, 1]
warmup_config = {
    'auto_withdrawal': False,
    'chain': 'ZKSYNC',
    'swap_amount_interval': '',
    'token': 'ETH',
    'amount_of_warmups': {'from': 3, 'to': 6},
    # 'allowed_dex': ['syncswap','woofi','lineaswap','horizondex''pancake'], #LINEA
    # 'allowed_dex': ['syncswap','mute','woofi','maverick','velocore','pancake','odos'] #ZKSYNC
    # 'allowed dex': ['woofi', 'maverick', 'horizondex', 'pancake', 'odos'] #BASE
    'allowed_dex': ['syncswap', 'mute', 'woofi', 'maverick', 'velocore', 'lineaswap', 'horizondex', 'pancake', 'odos'],
    'warmup_pairs': ['ETH>USDC'],  # specify for warming up by concrete pair
    # 'warmup_pairs': ['ETH>USDC', 'ETH>USDT'],
    'sell_tokens': True,
    'pair_sleep_interval': {'from': 60 * 15, 'to': 60 * 30}  # Interval between different tokens warm up of 1 account
}

# Module [BUNGEE, 11] Bungee [ZKSYNC, ARBITRUM, BSC, BASE, AVALANCHE, POLYGON, OPTIMISM]
bungee_config = {
    'from_chain': 'BASE',
    'to_chain': 'ZKSYNC',
    'refuel_amount_interval': 'max'
    # ['{'from': 0.001, 'to': 0.002}', '', 'max']
}

# Module [STARGATE_AUTO_BRIDGE, 13] Stargate auto bridge
stargate_auto_bridge_config = {
    'bridge_amount_interval': 'all_balance',
    'bridge_token': 'ETH',
    'end_network': '',  # name of network where all tokens will be collected
    'retry_interval': {'from': 60 * 15, 'to': 60 * 30},
    'sleep_interval': {'from': 60 * 30, 'to': 60 * 60},
    'only_end_network_bridge': False,  # if True, then only bridge will be to END NETWORK
    'original_chain_sequence': ['ARBITRUM', 'LINEA', 'BASE']
}

# Module [STARGATE_FARMING, 14] Stargate farming
stargate_farming_config = {
    'chain': 'BASE',  # ['BASE, 'LINEA']
    'bridge_amount_interval': 'all_balance',
    'all_balance_mode': 0,
    # 0 - all_balance игнорирует 'chain' и стекает ETH в чейне с максимально большим балансом
    # 1 - all_balance стекает весь баланс указанного 'chain'
    'operation': {'ADD_TO_POOL', 'DEPOSIT'},
    'token': 'ETH',
    'operation_sleep_interval': {'from': 60 * 2, 'to': 60 * 10}
    # 'operation': {'WITHDRAW', 'REDEEM_FROM_POOL'}
}

# Module [SELL_ALL, 2] This module sells all to ETH or USDC
sell_all_config = {
    'chain': 'ZKSYNC',
    'receive_token': 'ETH',  # [USDC, ETH]
    'sleep_interval': {'from': 60 * 30 * 1, 'to': 60 * 60 * 1}  # Interval between selling different tokens of 1 account
}

# Module [CONCRETE_SWAP, 3] Swap concrete pair in concrete DEX
swap_config = {
    'chain': 'LINEA',
    'app': 'syncswap',
    'from_token': 'ETH',
    'to_token': 'USDT',
    'amount_interval': {'from': 0.001, 'to': 0.001}
}

# Module [WRAPPING, 4] [Wrap/Unwrap WETH]
wrap_config = {
    'chain': 'ZKSYNC',
    'action': 'UNWRAP',  # [WRAP, UNWRAP]
    'amount_interval': 'all_balance'
}

# Module [SEND_TO_CEX, 6] Config
send_to_cex_config = {
    'chain': 'ZKSYNC',
    'amount': {'from': 1, 'to': 1},
    'token': 'NATIVE'  # [NATIVE, USDC]
}

# Module [DMAIL, 5] Config
send_dmail_config = {
    'chain': 'ZKSYNC',  # ['ZKSYNC', 'LINEA', 'SCROLL']
    'email_amount': {'from': 3, 'to': 6}
}

# Module [NFT_MINTER, 15] Все NFT пока работают только для ZKSYNC ["TAVAERA", "ZKS_DOMAIN", "ERA_DOMAIN",
# "EMPTY_NFT", "OMNI_SEA", "MERKLY_MINTER", "L2_TELEGRAPH_MINTER", "KREATOR_LAND", "ABBYSWORLD"]
nft_minter = {
    'nft_amount_interval': {'from': 1, 'to': 1},
    'nft_modules': ["SATOSHI_UNIVERSE"],
    'unique_nft': True # Currently only works for linea
}

# Module [LAYERBANK, 23]
layerbank = {
    'action': 'DEPOSIT',
    'sleep_interval': {'from': 60 * 15, 'to': 60 * 45},
}

# Module [LIQUIDITY_POOL, 24]
liquidity_pool = {
    'action': 'DEPOSIT+WITHDRAW',
    'amount': 'all_balance',
    'token': 'ETH',
    'sleep_interval': {'from': 60 * 2, 'to': 60 * 5},
    'dex': 'velocore',
    'chain': 'LINEA'
}

# Module [ZKDX, 25]
zkdx_config = {
    'amount_interval': 'all_balance',  # USDC amount
    'action': 'DEPOSIT'
}

# Module [CEX_SUB_ACCOUNT_TRANSFER, 26]
cex_account_transfer_config = {

}

# Module [MERKLY_REFUEL, 28] Bungee [ZKSYNC, ARBITRUM, ARBITRUM NOVA, BASE, LINEA, OPTIMISM, SCROLL]
merkly_refuel_config = {
    'start_chain': 'ZKSYNC',  # Если '', первый чейн будет RANDOM
    'end_chain': 'ZKSYNC',  # Если '', то последний чейн будет RANDOM
    'chains': ['ZKSYNC', 'ARBITRUM', 'BASE'],  # Список всех чейнов для которых делать Refuel.
    # Должен вмещать в себя start_chain и end_chain
    'refuel_amount_interval': {'from': 0.00005, 'to': 0.00009},
    'sleeping_interval': {'from': 60 * 5, 'to': 60 * 10}
}

# Module [CEX_WITHDRAW, 29] [ZKSYNC, LINEA, BASE, ARBITRUM, COREDAO, POLYGON, X_LAYER]
cex_withdraw = {
    'chain': 'ZKSYNC',
    'withdraw_interval': {'from': 0.01, 'to': 0.015},
    'min_auto_withdraw_interval': {'from': 0.1, 'to': 0.2},
    'token': 'NATIVE',  # [NATIVE, USDC, USDT]
    'cex': 'okx'
}

# Module [COREDAO_BRIDGE_AUTO, 31
coredao_bridge_auto_config = {
    'from_chain': 'COREDAO',
    'to_chain': 'POLYGON',
    'repeats': {'from': 2, 'to': 4},
    'bridge_amount_interval': 'all_balance',
    'sleep_interval': {'from': 60 * 25, 'to': 60 * 40},  # Интервал from_chain -> to_chain и to_chain -> from_chain
    'token': 'USDT'  # [USDC, USDT]
}

# Module [LAYER2_20, 32]
layer_2_20_config = {
    'chain': 'ZKSYNC',
    'to_chains': ['OPTIMISM', 'BASE', 'LINEA', 'ARBITRUM', 'SCROLL'],
    'repeats': {'from': 2, 'to': 4},
}

# Module [BANKING, 34]
banking_config = {
    'bank': 'MENDI_FINANCE',
    'actions': 'SUPPLY', # ['SUPPLY', 'BORROW', 'REPAY', 'REDEEM']
    'supply_token': 'WETH',
    'borrow_token': 'WETH',
    'sleep_interval': {'from': 1 * 5, 'to': 1 * 10},
    'chain': 'LINEA'
}

# Module [CLAIMER, 35]
claimer_config = {
    'chain': 'ARBITRUM',
    'token': 'NATIVE', # NATIVE or USDT or USDC
    'project': 'RABBY'
}

sleep_module_config = {}

# Module [ADS_IMPORT_PROXY, 36]
ads_import_config = {
    'ads_url': 'http://local.adspower.net:50325/api/v1/user'
}

# Module [BRIDGE, 37]
bridge_module = {
    'bridge': 'ORBITER',
    'bridge_amount_interval': 'all_balance',
    'from_chain': 'POLYGON',
    'to_chain': 'ZKSYNC',
    'token': 'USDT'
}

# Module [SMART_CONTRACT_DEPLOYMENT, 38]
smart_contract_deployment = {
    'chain': 'BASE'
}

erc20_balance = {
    'token_address': '0xd83af4fbD77f3AB65C3B1Dc4B38D7e67AEcf599A',
    'chain': 'LINEA'
}


from sybil_engine.contract.contract import Contract
from sybil_engine.data.networks import get_ids_chain
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/merkly.json")


class L2TelegraphMinter(Contract):
    def __init__(self, contract_address, web3):
        self.weth_token = get_tokens_for_chain(get_ids_chain()[web3.eth.chain_id])['WETH']
        super().__init__(contract_address, web3, abi)

    def mint(self, account):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = Web3.to_wei(0.0005, "ether")
        contract_txn = self.contract.functions.mint().build_transaction(txn_params)

        return contract_txn



from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.module.module import Module
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.gmx_reward_router import GmxRewardRouter


class GMXRewardRouter(Module):
    module_name = 'GMX_REWARD_ROUTER'
    module_config = None

    def execute(self, account, chain='ARBITRUM'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, None)

        contract_address = get_contracts_for_chain(chain_instance['chain'])['GMX_REWARD_ROUTER']
        gmx_reward_router = GmxRewardRouter(contract_address, web3)

        token = Erc20Token(chain, '0x1aDDD80E6039594eE970E5872D247bf0414C8903', web3)
        gmx_reward_router.unstake_and_redeem(account, token.balance(account).wei)

    def log(self):
        return "GMX REWARD WITHDRAW"



from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance_utils import get_native_balance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.utils import ModuleException
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.stargate_router import StargateRouter
from web3_wizzard_lib.core.contract.stargate_router_eth import StargateRouterEth
from web3_wizzard_lib.core.modules.stargate.stargate_balance_utils import StargatePoolToken


def add_to_pool(account, amount_to_pool, bridge_token, chain_instance, web3):
    def get_router_details(token_type, chain_inst):
        contracts = get_contracts_for_chain(chain_inst['chain'])
        if token_type == 'ETH':
            return StargateRouterEth(contracts["STARGATE_ROUTER_ETH"], web3), None
        else:
            erc20 = Erc20Token(chain_inst['chain'], token_type, web3)

            if erc20.allowance(account, contracts["STARGATE_ROUTER"]) < erc20.balance(account).wei:
                erc20.approve(account, contracts["STARGATE_ROUTER"])
            return StargateRouter(contracts["STARGATE_ROUTER"], web3), erc20

    logger.info(f"======= Adding liquidity {bridge_token} {chain_instance['chain']} ::: {account.address} =======")

    native_balance = get_native_balance(account, web3, chain_instance)
    logger.info(f"Native balance: {native_balance}")

    stargate_router, erc20 = get_router_details(bridge_token, chain_instance)

    balance = native_balance if bridge_token == 'ETH' else erc20.balance(account)
    if bridge_token != 'ETH':
        logger.info(f"Balance: {balance}")

    verify_balance(amount_to_pool, balance, native_balance)

    func = stargate_router.add_liquidity_eth if bridge_token == 'ETH' else stargate_router.add_liquidity

    logger.info(f"Pooling: {amount_to_pool}")
    func(account, amount_to_pool)


def verify_balance(amount_to_pool, balance, erc20_token):
    if amount_to_pool.wei > balance.wei:
        raise Exception(f"Account balance ({amount_to_pool}) is lower than amount to pool.")

    if balance.wei == 0:
        logger.info(f"{erc20_token.token} balance is 0")
        raise ModuleException("")


def find_max_pool_chain(balances):
    max_usdc_balance = max(balances, key=lambda x: x[1].wei)

    if max_usdc_balance[1] == 0:
        raise Exception("Can't bridge tokens, all chain USDC balances are zero")

    return max_usdc_balance


def get_pool_token_name(token_type):
    return 'STARGATE_ETH_POOL' if token_type == 'ETH' else 'STARGATE_USDT_POOL'


def get_pool_balances(token, account, chains):
    return [get_pool_balance_for_chain(chain, token, account) for chain in chains]


def get_pool_balance_for_chain(chain, pool_token_name, account):
    web3 = init_web3(get_chain_instance(chain), account.proxy)
    pool_token_instance = StargatePoolToken(chain, get_pool_token_name(pool_token_name), web3)
    return chain, pool_token_instance.balance(account), web3



from loguru import logger
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.domain.balance.tokens import Erc20Token

from web3_wizzard_lib.core.contract.mendi_token import MendiTokenContract
from web3_wizzard_lib.core.modules.bank.bank import Bank


class MendiFinance(Bank):
    app_name = 'MENDI_FINANCE'
    supported_chains = ['LINEA']

    def __init__(self, chain, web3):
        self.chain = chain
        self.web3 = web3

    def supply(self, account, amount):
        erc20 = Erc20Token(self.chain, amount.token, self.web3)
        amount = erc20.balance(account)

        me_token = f'me{amount.token}'
        mendi_token_address = get_tokens_for_chain(self.chain)[me_token]

        logger.info(f"Mendi supply {amount} {amount.token}")

        if erc20.allowance(account, mendi_token_address) < erc20.balance(account).wei:
            erc20.approve(account, mendi_token_address)

        mendi = MendiTokenContract(mendi_token_address, self.web3)
        mendi.mint(account, amount.wei)

    def borrow(self, account, amount):
        logger.info(f"Mendi borrow {amount.token}")

    def repay_borrow(self, account, amount):
        logger.info(f"Mendi repay {amount.token}")

    def redeem(self, account, amount):
        me_token = f'me{amount.token}'
        mendi_token_address = get_tokens_for_chain(self.chain)[me_token]
        mendi = MendiTokenContract(mendi_token_address, self.web3)

        redeem_balance = mendi.balance_of(account)

        if redeem_balance > 1000000000:
            logger.info(f"Mendi reedem {amount. token}")
            mendi.redeem(account, redeem_balance)
        else:
            logger.info(f"Mendi balance is 0")

    def get_deposit_amount(self, account, token):
        #me_token = f'meweETH'
        mendi_token_address = get_tokens_for_chain(self.chain)[token]
        mendi = MendiTokenContract(mendi_token_address, self.web3)

        return mendi.balance_of(account)




import random

from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.omnisea import OmniSea
from web3_wizzard_lib.core.utils.sub_module import SubModule


class OmniSeaModule(SubModule):
    module_name = 'OMNI_SEA'

    def execute(self, account, chain='ZKSYNC'):

        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['OMNISEA']
        omnisea = OmniSea(contract_address, web3)

        title, symbol = self.generate_collection_data()

        omnisea.create(account, title, symbol)

    @staticmethod
    def generate_collection_data():
        title = "".join(random.sample([chr(i) for i in range(97, 123)], random.randint(5, 15)))
        symbol = "".join(random.sample([chr(i) for i in range(65, 91)], random.randint(3, 6)))
        return title, symbol

    def log(self):
        return "OMNI SEA COLLECTION"



from typing import Union

import requests
from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance_utils import get_native_balance, interval_to_eth_balance, from_wei_to_eth
from sybil_engine.module.module import Module
from sybil_engine.utils.utils import ConfigurationException
from sybil_engine.utils.validation_utils import validate_chain, validate_refuel_interval
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.bungee import bungee_chain_ids, Bungee


class BungeeRefuel(Module):
    module_name = 'BUNGEE'
    module_config = 'bungee_config'

    def execute(self, refuel_amount_interval, from_chain, to_chain, account):
        from_chain_instance = get_chain_instance(from_chain)
        web3 = init_web3(from_chain_instance, account.proxy)

        bungee_contract = get_contracts_for_chain(from_chain)['BUNGEE']
        bungee = Bungee(bungee_contract, web3)

        try:
            limits = self.get_bungee_limits(from_chain)

            to_chain_limits = [
                chain for chain in limits if chain["chainId"] == bungee_chain_ids[to_chain] and chain["isEnabled"]
            ]

            if to_chain_limits:
                min_amount = float(from_wei_to_eth(int(to_chain_limits[0]["minAmount"])))
                max_amount = float(from_wei_to_eth(int(to_chain_limits[0]["maxAmount"])))

                if refuel_amount_interval == 'max':
                    refuel_amount_interval = {'from': max_amount * 0.9, 'to': max_amount * 0.98}

                if refuel_amount_interval == '':
                    refuel_amount_interval = {'from': min_amount, 'to': max_amount}

                if refuel_amount_interval['from'] < min_amount:
                    raise ConfigurationException(
                        f'Min refuel amount for {from_chain} is {min_amount} < {refuel_amount_interval["from"]}')

                if refuel_amount_interval['to'] > max_amount:
                    raise ConfigurationException(
                        f'Max refuel amount for {from_chain} is {max_amount} > {refuel_amount_interval["to"]}')

                native_without_min = get_native_balance(account, web3, from_chain_instance).minus(
                    self.min_native_balance)

                amount = interval_to_eth_balance(refuel_amount_interval, account, from_chain, web3)

                if amount.wei > native_without_min.wei:
                    amount = native_without_min

                logger.info(f"Refuel {from_chain.title()} > {to_chain.title()} | {amount}")

                bungee.refuel(account, to_chain, amount.wei)
            else:
                logger.info("skip")
        except Exception as e:
            logger.error(f"Bungee refuel error | error {e}")

    def get_bungee_limits(self, from_chain) -> Union[dict, bool]:
        bungee_data = self.get_bungee_data()

        try:
            if from_chain == 'ZKSYNC':
                limits = [chain_data for chain_data in bungee_data if chain_data["name"] == "zkSync"][0]["limits"]
            elif from_chain == 'ARBITRUM':
                limits = [chain_data for chain_data in bungee_data if chain_data["name"] == "Arbitrum"][0]["limits"]
            elif from_chain == 'OPTIMISM':
                limits = [chain_data for chain_data in bungee_data if chain_data["name"] == "Optimism"][0]["limits"]
            elif from_chain == 'POLYGON':
                limits = [chain_data for chain_data in bungee_data if chain_data["name"] == "Polygon"][0]["limits"]
            elif from_chain == 'AVALANCHE':
                limits = [chain_data for chain_data in bungee_data if chain_data["name"] == "Avalanche"][0]["limits"]
            elif from_chain == 'BSC':
                limits = [chain_data for chain_data in bungee_data if chain_data["name"] == "BSC"][0]["limits"]
            elif from_chain == 'BASE':
                limits = [chain_data for chain_data in bungee_data if chain_data["name"] == "Base"][0]["limits"]
            else:
                raise Exception("Not supported")

            return limits
        except Exception as e:
            return False

    def get_bungee_data(self):
        url = "https://refuel.socket.tech/chains"
        response = requests.get(url)
        if response.status_code == 200:
            data = response.json()["result"]
            return data
        return False

    def log(self):
        return "BUNGEE REFUEL"

    def parse_params(self, module_params):
        validate_chain(module_params['from_chain'])
        validate_chain(module_params['to_chain'])
        validate_refuel_interval(module_params['refuel_amount_interval'])

        return module_params['refuel_amount_interval'], module_params['from_chain'], module_params['to_chain']



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction

with open("resources/abi/stargate_farming.json") as f:
    abi = f.read()


class StargateFarming(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def deposit(self, account, pool_amount):
        sender = account.address
        txn_params = self.build_generic_data(sender, False)

        contract_txn = self.contract.functions.deposit(0, pool_amount.wei).build_transaction(txn_params)
        contract_txn['gas'] = int(self.web3.eth.estimate_gas(contract_txn) * 1.05)

        return contract_txn

    @evm_transaction
    def withdraw(self, account, amount_to_bridge):
        sender = account.address

        txn_params = self.build_generic_data(sender, False)

        contract_txn = self.contract.functions.withdraw(
            0,
            amount_to_bridge
        ).build_transaction(txn_params)
        contract_txn['gas'] = int(self.web3.eth.estimate_gas(contract_txn) * 1.05)

        return contract_txn

    def user_info(self, account):
        return int(self.contract.functions.userInfo(0, account.address).call()[0])



from sybil_engine.contract.send import Send
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei
from sybil_engine.utils.app_account_utils import AppAccount

from web3_wizzard_lib.core.modules.nft.nft_submodule import NftSubmodule


class LineaCultureWeek4Day2Approve(NftSubmodule):
    module_name = 'LINEA_CULTURE_4_2_APPROVE'
    nft_address = '0x3f0A935c8f3Eb7F9112b54bD3b7fd19237E441Ee'

    def execute(self, account: AppAccount, chain='LINEA', wei_cost=from_eth_to_wei(0)):
        data = f"0xa22cb4650000000000000000000000000cab6977a9c70e04458b740476b498b2140196410000000000000000000000000000000000000000000000000000000000000001"

        Send(
            None,
            self.create_web3(account, chain)
        ).send_to_wallet(
            account,
            self.nft_address,
            NativeBalance(wei_cost, chain, "ETH"),
            data
        )

    def log(self):
        return "LINEA CULTURE 4 WEEK DAY 2 (Borja Moskv APPROVE)"



from loguru import logger
#from pyuseragents import random as random_ua
from requests import Session
from sybil_engine.utils.retry import retry


class IntractAPI:
    def __init__(self, account):
        self.account = account
        self.session = Session()
        self.session.headers['user-agent'] = random_ua()
        if account.proxy is not None and account.proxy != '':
            self.session.proxies.update({'http': account.proxy, 'https': account.proxy})
        else:
            logger.warning('You are not using proxy')

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def intract_get_nonce(self, address: str):
        payload = {'walletAddress': address}
        url = 'https://api.intract.io/api/qv1/auth/generate-nonce'

        self.session.headers.clear()
        r = self.session.post(url, json=payload)

        if r.json().get('success') == True:
            return r.json()['data']['nonce']
        else:
            logger.debug(f'nonce intract response: {r.status_code} {r.reason} | {r.text}')
            raise Exception(f'Couldnt generate nonce for Intract')

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def intract_login(self, address: str, signature: str):
        payload = {
            "signature": signature,
            "userAddress": address,
            "chain": {"id": 59144, "name": "Linea", "network": "Linea",
                      "nativeCurrency": {"decimals": 18, "name": "Ether", "symbol": "ETH"},
                      "rpcUrls": {"public": {"http": ["https://linea.drpc.org"]}, "default": {
                          "http": ["https://linea-mainnet.infura.io/v3/bfc263a4f3cf49998641d16c24fd0b46"]}},
                      "blockExplorers": {"etherscan": {"name": "Lineascan", "url": "https://lineascan.build/"},
                                         "default": {"name": "Lineascan", "url": "https://lineascan.build/"}},
                      "unsupported": False},
            "isTaskLogin": False,
            "width": "590px",
            "reAuth": False,
            "connector": "metamask",
            "referralCode": None,
            "referralLink": None,
            "referralSource": None
        }
        url = 'https://api.intract.io/api/qv1/auth/wallet'

        r = self.session.post(url, json=payload)

        if r.json().get('isEVMLoggedIn') == True:
            self.user_id = self.get_user()['_id']

            self.session.headers.update({
                'Cookies': r.headers["Set-Cookie"].split(';')[0],
                'Questuserid': self.user_id,
            })
            return True
        else:
            logger.debug(f'login intract response: {r.status_code} {r.reason} | {r.text}')
            raise Exception(f'Couldnt login Intract')

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def intract_get_tasks(self, compaignID):
        self.active_tasks = []

        url = f'https://api.intract.io/api/qv1/campaign/{compaignID}'
        r = self.session.get(url)

        wave = r.json()
        # for wave in r.json():
        if wave["status"] == 'ACTIVE' and wave.get('locked') != True: self.active_tasks.append(wave["name"])

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def set_wallet(self, address: str):
        try:
            if self.get_user().get('lineaWalletAddress'):
                return True

            payload = {
                "userId": self.user_id,
                "lineaWalletAddress": address
            }
            url = 'https://api.intract.io/api/qv1/linea/user/set-wallet'
            r = self.session.post(url, json=payload)

            if r.json().get('message') == "Linea wallet address updated successfully":
                logger.info(f'Wallet successfully linked')
                return True
        except Exception as err:
            raise Exception(f'Couldnt set wallet: {err} | {r.text}')

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def intract_streak(self):
        try:
            url = 'https://api.intract.io/api/qv1/linea/user/streak'
            r = self.session.post(url, json={})

            streakCount = r.json().get('streakCount')
            isFirstTimeMarked = r.json().get('isFirstTimeMarked')

            if streakCount:
                logger.info(f'New GM streak: {streakCount + 1}')
            elif isFirstTimeMarked:
                logger.info(f'New GM streak: 1')
            elif 'Linea streak already done for today' in r.text:
                logger.info(f'New GM streak: already did streak today')
            else:
                raise Exception(f'bad response {r.status_code} {r.reason}')
        except Exception as err:
            if 'Internal server error' in r.text:
                logger.error(f'Couldnt use GM streak')
            else:
                logger.error(f'Couldnt post streak: {err} | {r.text}')

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def get_user(self):
        try:
            url = 'https://api.intract.io/api/qv1/auth/get-user?projectId=660c1fa77851c55d93a1c0e0'
            r = self.session.get(url)
            return r.json()

        except Exception as err:
            raise Exception(f'Couldnt get user: {err} | {r.text}')

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def get_super_user(self):
        try:
            # self.intract_streak()

            url = 'https://api.intract.io/api/qv1/auth/get-super-user'
            r = self.session.get(url)

            return {
                'achieves': len(r.json().get('badges')),
                'streak': r.json().get('gmStreak')['streakCount'],
                'gems': r.json().get('totalGems'),
                'xp': r.json().get('totalXp'),
                'super_user_id': r.json().get('_id'),
            }

        except Exception as err:
            raise Exception(f'Couldnt get super user: {err} | {r.text}')

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def claim_achievements(self):
        try:
            url = 'https://api.intract.io/api/qv1/achievement/init'
            r = self.session.post(url)

            for achieve in r.json():
                if achieve["isClaimed"] == False and achieve["isCompleted"] == True:
                    try:
                        payload = {
                            "achievementIds": [achieve["achievementId"]]
                        }
                        url = 'https://api.intract.io/api/qv1/achievement/claim'
                        r = self.session.post(url, json=payload)

                        if 'User achievement reward already claimed' in r.text or 'User achievement not found' in r.text:
                            pass
                        else:
                            logger.info(f'Claimed achievement: {r.json()[0]["name"]}')
                    except Exception as err:
                        logger.error(
                            f'Claim achievement {achieve["achievementId"]} {achieve["name"]} error: {err} | {r.text}')

        except Exception as err:
            logger.error(f'Couldnt fetch achievements: {err} | {r.text}')

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def activate_journey(self, campaign_id: str):
        url = f'https://api.intract.io/api/qv1/journey/fetch?campaignId={campaign_id}&channelCode=DEFAULT&referralCode=null'

        r = self.session.get(url)
        if r.json().get('isActive') != True:
            logger.error(f'Activate journey error: {r.text}')

        return r.json()

    @retry(max_attempts=10, retry_interval={'from': 10, 'to': 20})
    def verify_quest(self, task):
        data = {
            "campaignId": task['campaignId'],
            "userInputs": {},
            "userVerificationData": {},
            "taskId": task['_id']
        }

        headers = {
            'questuserid': self.session.headers['Questuserid'],
            'authorization': f'Bearer {self.session.headers["Cookies"].split('=')[1]}'
        }

        url = 'https://api.intract.io/api/qv1/task/verify-v2'
        r = self.session.post(url, json=data, headers=headers)

        logger.info(r.json())

        if r.json()['verified'] == 'False':
            raise Exception("Quest wasn't verified")

        return r.status_code

    def set_primary_task_identity(self, account):
        url = 'https://api.intract.io/api/qv1/auth/set-primary-task-identity'
        payload = {"identity": account.address, "namespaceTag": "EVM"}
        r = self.session.post(url, json=payload)



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction, l0_evm_transaction

from web3_wizzard_lib.core.modules.stargate.stargate_gas_checker import stargate_check_gas

with open("resources/abi/stargate_swapETH_abi.json") as f:
    abi = f.read()


class StargateRouterEth(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def add_liquidity_eth(self, account, pool_amount):
        from_address = account.address

        txn_params = self.build_generic_data(from_address, False)
        txn_params['value'] = pool_amount.wei

        contract_txn = self.contract.functions.addLiquidityETH().build_transaction(txn_params)

        return contract_txn

    @stargate_check_gas(token='ETH')
    @l0_evm_transaction
    def swap_eth(self, account, value_wei, amount_to_bridge_wei, to_chain_instance):
        from_address = account.address

        txn_params = self.build_generic_data(account.address, False)
        txn_params['value'] = value_wei

        contract_txn = self.contract.functions.swapETH(
            to_chain_instance['stargate_chain_id'],
            from_address,
            from_address,
            amount_to_bridge_wei,
            int(amount_to_bridge_wei * 0.985)
        ).build_transaction(txn_params)

        return contract_txn



import random

from loguru import logger
from sybil_engine.contract.send import Send
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import get_native_balance
from sybil_engine.module.module import Order, RepeatableModule
from sybil_engine.utils.utils import interval_to_int, ConfigurationException
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules.bridge.orbiter_sub import bridge_codes


class Layer2_20(RepeatableModule):
    module_name = 'Layer2_20'
    allowed_chains = ['ZKSYNC', 'OPTIMISM', 'BASE', 'LINEA', 'ARBITRUM', 'SCROLL']
    random_order = Order.RANDOM
    repeat_conf = 'repeats'
    module_config = 'layer_2_20_config'

    @RepeatableModule.repeatable_log
    def execute(self, chains, to_chains, account):
        chain = random.choice(chains)
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        temp_to_chains = [item for item in to_chains if item != chain]
        if not temp_to_chains:
            raise ConfigurationException("No chains to send to")
        to_chain = random.choice(temp_to_chains)

        layer2_20_contract_address = get_contracts_for_chain(chain)["LAYER2_20"]

        send = Send(layer2_20_contract_address, web3)

        amount_to_send = NativeBalance(
            230000000000000 + bridge_codes[to_chain],
            chain,
            chain_instance['gas_token']
        )
        data = '0x646174613a2c7b2270223a226c61796572322d3230222c226f70223a22636c61696d222c227469636b223a22244c32222c22616d74223a2231303030227d'

        native_balance = get_native_balance(account, web3, chain_instance)

        logger.info(f"Native {chain} balance: " + str(native_balance))
        logger.info(f"Inscription {chain} -> {to_chain} | {amount_to_send}")

        send.send_to_wallet(account, layer2_20_contract_address, amount_to_send, data)

    def log(self):
        return "LAYER2_20"

    def parse_params(self, module_params):
        return module_params['chains'], module_params['to_chains']

    def order(self):
        return Order.RANDOM

    def repeats(self, module_params):
        if self.repeat_conf not in module_params:
            return 1
        else:
            return range(interval_to_int(module_params[self.repeat_conf]))



import requests
from loguru import logger
from sybil_engine.module.module import Module


class OrbiterChecker(Module):
    module_name = 'ORBITER_CHECKER'
    module_config = None

    total_points = 0

    def execute(self, account):
        url = f'https://api.orbiter.finance/points_platform/rank/address/{account.address.lower()}'

        response = requests.get(url)

        points = int(response.json()['result']['point'])

        if points > 0:
            self.total_points = self.total_points + points

        logger.info(f"Points {points}")
        logger.info(f"Total points {self.total_points}")

    def log(self):
        return "ORBITER CHECKER"



from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.snap_contract import SnapContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class SnapBonus(SubModule):
    module_name = 'SNAP_BONUS'
    allow_reuse_address = True

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['SNAP']
        social_scan = SnapContract(contract_address, web3)

        balance = social_scan.contract.functions.balanceOf(account.address).call()
        token_id = social_scan.contract.functions.tokenOfOwnerByIndex(account.address, 0).call()

        if balance == 0:
            logger.error(f"[{account.app_id}][{account.address}] Not found NFT. Skip module")
            return

        social_scan.stake(account, token_id)

    def log(self):
        return "SNAP_BONUS MINT NFT"



from sybil_engine.utils.utils import ConfigurationException

from web3_wizzard_lib.core.modules.bridge.bridge import Bridge
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NotEnoughNativeBalance
from sybil_engine.domain.balance.balance_utils import interval_to_native_balance, verify_balance

from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.eth_scroll_bridge_contract import EthScrollBridgeContract
from web3_wizzard_lib.core.contract.scroll_bridge_contract import ScrollBridgeContract
from web3_wizzard_lib.core.contract.scroll_oracle import ScrollOracle

class ScrollBridgeSubModule(Bridge):

    def execute(self, bridge_amount_interval, from_chain, to_chain, token, min_native_balance, account):
        from_chain_instance = get_chain_instance(from_chain)
        web3 = init_web3(from_chain_instance, account.proxy)
        native_without_min = verify_balance(min_native_balance, from_chain_instance, account, web3)
        amount = interval_to_native_balance(bridge_amount_interval, account, from_chain_instance['chain'], web3)
        if amount.wei > native_without_min.wei:
            if bridge_amount_interval == 'all_balance':
                amount = native_without_min
            else:
                raise NotEnoughNativeBalance(
                    f"The account balance ({native_without_min}) < bridging amount ({amount}).")
        if from_chain == 'ETH_MAINNET':
            contract_oracle = get_contracts_for_chain(from_chain)["SCROLL_ORACLE"]
            fee = ScrollOracle(contract_oracle, web3).estimateCrossDomainMessageFee()

            scroll_bridge_contract_address = get_contracts_for_chain(from_chain)["SCROLL_BRIDGE"]
            scroll_bridge = EthScrollBridgeContract(scroll_bridge_contract_address, web3)

            scroll_bridge.sendMessage(account, amount.wei, fee)
        elif from_chain == 'SCROLL':
            scroll_bridge_contract_address = get_contracts_for_chain(from_chain)["SCROLL_BRIDGE"]
            scroll_bridge = ScrollBridgeContract(scroll_bridge_contract_address, web3)

            scroll_bridge.withdraw(account, amount.wei)
        else:
            raise ConfigurationException(f"{from_chain} chain is unsupported, only ETH_MAINNET and SCROLL allowed")


from sybil_engine.domain.dex import Dex

from ...contract.izumi import IzumiRouter


class Izumi(Dex):
    dex_name = 'izumi'
    swap_contract = 'IZUMI'
    supported_chains = ['LINEA']

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.weth_address = self.tokens['WETH']
        self.izumi = IzumiRouter(self.chain_contracts[self.swap_contract], self.web3)

    def swap_token_for_native(self, account, amount_to_swap, from_token_address, slippage):
        amount_out_min = int(self.get_amount_out_min(amount_to_swap, from_token_address, self.weth_address) * slippage)
        args = [account, amount_to_swap, amount_out_min, from_token_address, self.weth_address]
        func = self.izumi.multicall

        return args, func

    def swap_native_for_token(self, account, amount_to_swap, slippage, to_token_address):
        amount_out_min = int(self.get_amount_out_min(amount_to_swap, self.weth_address, to_token_address) * slippage)
        args = [account, amount_to_swap, amount_out_min, self.weth_address, to_token_address]
        func = self.izumi.multicall

        return args, func

    def swap_token_for_token(self, account, amount_to_swap, slippage, from_token_address, to_token_address):
        raise Exception("Not supported yet")

    def get_amount_out_min(self, amount_to_swap, from_token_address, to_token_address):
        izumi = self.chain_contracts["IZUMI"]
        izumi_router = IzumiRouter(izumi, self.web3)

        if amount_to_swap.token == 'ETH':
            return int((izumi_router.quote_price('weETH') * amount_to_swap.wei) * 0.999)
        else:
            return int((izumi_router.quote_price('ETH') * amount_to_swap.wei) * 0.999)




from sybil_engine.contract.send import Send
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei
from sybil_engine.utils.app_account_utils import AppAccount

from web3_wizzard_lib.core.modules.nft.nft_submodule import NftSubmodule


class LineaCultureWeek2Day2(NftSubmodule):
    module_name = 'LINEA_CULTURE_2_6'
    nft_address = '0x0841479e87Ed8cC7374d3E49fF677f0e62f91fa1'

    def execute(self, account: AppAccount, chain='LINEA', wei_cost=from_eth_to_wei(0)):
        Send(
            None,
            self.create_web3(account, chain)
        ).send_to_wallet(
            account,
            self.nft_address,
            NativeBalance(wei_cost, chain, "ETH"),
            "0x00000000"
        )

    def log(self):
        return "LINEA CULTURE 2 WEEK DAY 6 (Toad The Great)"



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.data.tokens import get_tokens_for_chain

from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/velocore_pool.json")


def to_bytes32(address):
    return Web3.to_bytes(hexstr=address).rjust(32, b'\0')


class VelocorePoolContract(Contract):
    ZERO = '0x0000000000000000000000000000000000000000000000000000000000000000'

    def __init__(self, contract_address, web3):
        self.LVC = get_tokens_for_chain('LINEA')['LVC']
        self.VLP = get_tokens_for_chain('LINEA')['VLP']
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def deposit(self, account, amount, deposit_token, pool):
        sender = account.address

        txn_params = self.build_generic_data(sender, set_contract_address=False)

        amount_hex = '0x' + format(amount, '064x')
        min_uint_256_hex = '00000000000000000000000000000000'
        inverted_amount = self.invert_int(amount)
        inverted_amount_hex = '0x' + min_uint_256_hex + inverted_amount

        if deposit_token == 'ETH':
            tokenRef = [
                to_bytes32(self.VLP),
                to_bytes32(self.LVC),
                to_bytes32('0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee')
            ]
            deposit = [0, 0, 0]

            txn_params['value'] = amount

            ops = [
                (
                    to_bytes32(f'0x040000000000000000000000{sender[2:]}'),
                    [to_bytes32('0x02000000000000000000000000000000' + format(amount, '032x'))],
                    to_bytes32(self.ZERO)
                ),
                (
                    to_bytes32(f'0x040000000000000000000000{sender[2:]}'),
                    [to_bytes32('0x02000000000000000000000000000000' + inverted_amount)],
                    to_bytes32(self.ZERO)
                ),
                (
                    to_bytes32(f'000000000000000000000000{pool[2:]}'),
                    [
                        to_bytes32('000100000000000000000000000000007fffffffffffffffffffffffffffffff'),
                        to_bytes32('020200000000000000000000000000007fffffffffffffffffffffffffffffff')
                    ],
                    to_bytes32(self.ZERO)
                ),
                (
                    to_bytes32('0500000000000000000000000000000000000000000000000000000000000000'),
                    [to_bytes32('00010000000000000000000000000000ffffffffffffffffffffffffff785a9c')],
                    to_bytes32(self.ZERO)
                ),
                (
                    to_bytes32(f'010000000000000000000000{pool[2:]}'),
                    [
                        to_bytes32('000200000000000000000000000000007fffffffffffffffffffffffffffffff'),
                        to_bytes32('0101000000000000000000000000000000000000000000000000000000000000')
                    ],
                    to_bytes32(self.ZERO)
                )
            ]
        else:
            wstETH = get_tokens_for_chain('LINEA')[deposit_token]
            tokenRef = [
                to_bytes32(wstETH),
                to_bytes32(self.LVC),
                to_bytes32(self.VLP),
                to_bytes32('0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee')
            ]
            deposit = [amount, 0, 0, 0]

            wstETHLP = '0x1D12E25e5516e5aD32F97fE9Edc332Bf4683f487'

            ops = [
                (
                    to_bytes32(f'0x040000000000000000000000{sender[2:]}'),
                    [to_bytes32(amount_hex)],
                    to_bytes32(self.ZERO)
                ),
                (
                    to_bytes32(f'0x040000000000000000000000{sender[2:]}'),
                    [to_bytes32(inverted_amount_hex)],
                    to_bytes32(self.ZERO)
                ),
                (
                    to_bytes32(f'000000000000000000000000{wstETHLP[2:]}'),
                    [
                        to_bytes32('000200000000000000000000000000007fffffffffffffffffffffffffffffff'),
                        to_bytes32('030100000000000000000000000000007fffffffffffffffffffffffffffffff')
                    ],
                    to_bytes32(self.ZERO)
                ),
                (
                    to_bytes32(f'000000000000000000000000{pool[2:]}'),
                    [
                        to_bytes32('020100000000000000000000000000007fffffffffffffffffffffffffffffff'),
                        to_bytes32('030200000000000000000000000000007fffffffffffffffffffffffffffffff')
                    ],
                    to_bytes32(self.ZERO)
                ),
                (
                    to_bytes32('0500000000000000000000000000000000000000000000000000000000000000'),
                    [to_bytes32('02010000000000000000000000000000ffffffffffffffffffffffb4ad12f9cd')],
                    to_bytes32(self.ZERO)
                ),
                (
                    to_bytes32(f'010000000000000000000000{pool[2:]}'),
                    [
                        to_bytes32('0101000000000000000000000000000000000000000000000000000000000000'),
                        to_bytes32('020200000000000000000000000000007fffffffffffffffffffffffffffffff')
                    ],
                    to_bytes32(self.ZERO)
                )
            ]

        contract_txn = self.contract.functions.execute(tokenRef, deposit, ops).build_transaction(txn_params)

        if self.web3.eth.chain_id == 59144:
            contract_txn['gas'] = contract_txn['gas'] * 2

        return contract_txn

    @evm_transaction
    def withdraw(self, account, eth_amount, eth_vlp, pool):
        sender = account.address

        txn_params = self.build_generic_data(sender, set_contract_address=False)

        tokenRef = [
            to_bytes32(self.LVC),
            to_bytes32(self.VLP),
            to_bytes32('0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee')
        ]
        deposit = [0, 0, 0]

        inverted_eth_amount = self.invert_int(eth_amount)
        inverted_eth_vlp = self.invert_int(eth_vlp[0])

        ops = [
            (
                to_bytes32(f'010000000000000000000000{pool[2:]}'),
                [
                    to_bytes32('0001000000000000000000000000000000000000000000000000000000000000'),
                    to_bytes32(f'01000000000000000000000000000000{inverted_eth_vlp}')
                ],
                to_bytes32('0x0000000000000000000000000000000000000000000000000000000000000000')
            ),
            (
                to_bytes32(f'000000000000000000000000{pool[2:]}'),
                [
                    to_bytes32('010200000000000000000000000000007fffffffffffffffffffffffffffffff'),
                    to_bytes32('020100000000000000000000000000007fffffffffffffffffffffffffffffff')
                ],
                to_bytes32('0x0000000000000000000000000000000000000000000000000000000000000000')
            ),
            (
                to_bytes32('0500000000000000000000000000000000000000000000000000000000000000'),
                [to_bytes32(f'02010000000000000000000000000000{inverted_eth_amount}')],
                to_bytes32('0x0000000000000000000000000000000000000000000000000000000000000000')
            )
        ]

        contract_txn = self.contract.functions.execute(tokenRef, deposit, ops).build_transaction(txn_params)

        if self.web3.eth.chain_id == 59144:
            contract_txn['gas'] = contract_txn['gas'] * 2

        return contract_txn

    def allPairs(self):
        return self.contract.functions.allPairs(1).call()

    def facets(self):
        return self.contract.functions.facets().call()

    def invert_int(self, amount):
        max_uint_256_hex = 'ffffffffffffffffffffffffffffffff'
        hex_2 = int(max_uint_256_hex, 16) - amount + 1
        hex_string = hex(hex_2)[2:].rjust(32, '0')

        return hex_string
# 5411000000000000
# 5410999999999999



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/zkdx_data.json")


class ZKDXDataContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)
        # self.zkdx_token = '0x176211869cA2b568f2A7D4EE941E073a821EE1ff'
        self.zkdx_token = '0x2167C4D5FE05A1250588F0B8AA83A599e7732eae'

    def get_balance(self, account):
        return self.contract.functions.getTokenBalances(account.address, [self.zkdx_token]).call()



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/yoddlo.json")


class YuliverseContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def purchase(self, account):
        txn_params = self.build_generic_data(account.address, True)

        function_selector = "0x064a181e"

        # Example uint256 paramet er
        varg0 = 123456789  # Make sure this is the value you intend to use

        # Example bytes parameter (signature) - Ensure this is exactly 65 bytes
        varg1 = b'Your65ByteSignatureHere'  # This should be the actual signature bytes

        # Encoding the transaction data
        data = function_selector
        data += varg0.to_bytes(32, byteorder='big').hex()
        data += (32).to_bytes(32, byteorder='big').hex()  # Offset for dynamic data
        data += (65).to_bytes(32, byteorder='big').hex()  # Length of the signature
        data += varg1.hex()

        print(data)

        # Concatenate the selector and the encoded parameters
        txn_params['data'] = data

        return txn_params



from sybil_engine.module.module import Order, RepeatableModule
from sybil_engine.utils.utils import interval_to_int, randomized_sleeping

from web3_wizzard_lib.core.modules.bridge.coredao_bridge_sub import CoreDaoBridgeSubModule


class CoreDaoBridgeAuto(RepeatableModule):
    module_name = 'COREDAO_BRIDGE_AUTO'
    allowed_chains = ['POLYGON', 'COREDAO']
    random_order = Order.RANDOM
    repeat_conf = 'repeats'
    module_config = 'coredao_bridge_auto_config'

    @RepeatableModule.repeatable_log
    def execute(self, from_chain, to_chain, bridge_amount_interval, token, sleep_interval, account):
        CoreDaoBridgeSubModule().execute(bridge_amount_interval, from_chain, to_chain, token, account)
        randomized_sleeping(sleep_interval)
        CoreDaoBridgeSubModule().execute(bridge_amount_interval, to_chain, from_chain, token, account)

    def log(self):
        return "COREDAO BRIDGE AUTO"

    def parse_params(self, module_params):
        return [
            module_params['from_chain'],
            module_params['to_chain'],
            module_params['bridge_amount_interval'],
            module_params['token'],
            module_params['sleep_interval'],
        ]

    def order(self):
        return Order.RANDOM

    def repeats(self, module_params):
        if self.repeat_conf not in module_params:
            return 1
        else:
            return range(interval_to_int(module_params[self.repeat_conf]))
